@@ -6,7 +6,7 @@
  * \____/\____/_/  |_\___/\___/\___/____/____/
  *
  * The MIT License (MIT)
- * Copyright (c) 2009-2017 Gerardo Orellana <hello @ goaccess.io>
+ * Copyright (c) 2009-2018 Gerardo Orellana <hello @ goaccess.io>
  */
 
 'use strict';
@@ -19,8 +19,9 @@ function truncate(text, width) {
 		var gw = $d3parent.node().getBBox();
 		var x = (Math.min(gw.width, width) / 2) * -1;
 		// adjust wrapper <svg> width
-		if ('svg' == parent.nodeName)
+		if ('svg' == parent.nodeName) {
 			$d3parent.attr('width', width).attr('x', x);
+		}
 		// wrap <text> within an svg
 		else {
 			$d3parent.insert('svg', function () {
@@ -42,7 +43,7 @@ function AreaChart(dualYaxis) {
 			top    : 20,
 			right  : 50,
 			bottom : 40,
-			left   : 50
+			left   : 50,
 		},
 		height = 170,
 		nTicks = 10,
@@ -78,8 +79,6 @@ function AreaChart(dualYaxis) {
 		.scale(yScale0)
 		.orient('left')
 		.tickFormat(function (d) {
-			if (format.y0)
-				return GoAccess.Util.fmtValue(d, format.y0);
 			return d3.format('.2s')(d);
 		});
 
@@ -249,7 +248,7 @@ function AreaChart(dualYaxis) {
 			.enter().append('text')
 			.attr('class', 'axis-label y0')
 			.attr('y', 10)
-			.attr('x', 50)
+			.attr('x', 53)
 			.text(labels.y0);
 
 		if (!dualYaxis)
@@ -262,7 +261,7 @@ function AreaChart(dualYaxis) {
 			.attr('y', 10)
 			.text(labels.y1);
 		dualYaxis && tEnter
-			.attr('x', width - 25)
+			.attr('x', width - 25);
 	}
 
 	function createSkeleton(svg) {
@@ -334,7 +333,7 @@ function AreaChart(dualYaxis) {
 				return pl + ' ' + pl;
 			})
 			.attr('stroke-dashoffset', function (d) {
-				return pathLen(d3.select(this))
+				return pathLen(d3.select(this));
 			});
 		// update
 		path
@@ -388,8 +387,8 @@ function AreaChart(dualYaxis) {
 			.attr('r', radius)
 			.attr('class', 'point');
 		points
-			.attr('cx', function (d) { return xScale(d[0]) })
-			.attr('cy', function (d) { return yScale0(d[1]) })
+			.attr('cx', function (d) { return xScale(d[0]); })
+			.attr('cy', function (d) { return yScale0(d[1]); });
 		// remove elements
 		points.exit().remove();
 
@@ -404,8 +403,8 @@ function AreaChart(dualYaxis) {
 			.attr('r', radius)
 			.attr('class', 'point');
 		points
-			.attr('cx', function (d) { return xScale(d[0]) })
-			.attr('cy', function (d) { return yScale1(d[2]) })
+			.attr('cx', function (d) { return xScale(d[0]); })
+			.attr('cy', function (d) { return yScale1(d[2]); });
 		// remove elements
 		points.exit().remove();
 	}
@@ -424,7 +423,7 @@ function AreaChart(dualYaxis) {
 
 		// Update the y0-axis.
 		g.select('.y0.axis')
-			.call(yAxis0.tickValues(getYTicks(yScale0))	);
+			.call(yAxis0.tickValues(getYTicks(yScale0)));
 
 		if (!dualYaxis)
 			return;
@@ -606,7 +605,7 @@ function BarChart(dualYaxis) {
 			top    : 20,
 			right  : 50,
 			bottom : 40,
-			left   : 50
+			left   : 50,
 		},
 		height = 170,
 		nTicks = 10,
@@ -642,8 +641,6 @@ function BarChart(dualYaxis) {
 		.scale(yScale0)
 		.orient('left')
 		.tickFormat(function (d) {
-			if (format.y1)
-				return GoAccess.Util.fmtValue(d, format.y1);
 			return d3.format('.2s')(d);
 		});
 
@@ -704,7 +701,7 @@ function BarChart(dualYaxis) {
 		xScale.domain(data.map(function (d) {
 			return d[0];
 		}))
-		.rangeBands([0, innerW()], .1);
+		.rangeBands([0, innerW()], 0.1);
 
 		// Update the y-scale.
 		yScale0.domain([0, d3.max(data, function (d) {
@@ -780,7 +777,7 @@ function BarChart(dualYaxis) {
 			.enter().append('text')
 			.attr('class', 'axis-label y0')
 			.attr('y', 10)
-			.attr('x', 50)
+			.attr('x', 53)
 			.text(labels.y0);
 
 		if (!dualYaxis)
@@ -793,7 +790,7 @@ function BarChart(dualYaxis) {
 			.attr('y', 10)
 			.text(labels.y1);
 		dualYaxis && tEnter
-			.attr('x', width - 25)
+			.attr('x', width - 25);
 	}
 
 	function createSkeleton(svg) {
@@ -850,7 +847,7 @@ function BarChart(dualYaxis) {
 		// update
 		bars
 			.attr('width', xScale.rangeBand() / 2)
-			.attr('x', function (d) { return xScale(d[0]) })
+			.attr('x', function (d) { return xScale(d[0]); })
 			.transition()
 			.delay(function (d, i) { return i / data.length * 1000; })
 			.duration(500)
@@ -862,7 +859,7 @@ function BarChart(dualYaxis) {
 		if (!dualYaxis)
 			return;
 
-		var bars = g.select('g.bars.y1').selectAll('rect.bar')
+		bars = g.select('g.bars.y1').selectAll('rect.bar')
 			.data(data);
 		// enter
 		bars
@@ -871,12 +868,12 @@ function BarChart(dualYaxis) {
 			.attr('class', 'bar')
 			.attr('height', 0)
 			.attr('width', function (d, i) { return xScale.rangeBand() / 2; })
-			.attr('x', function (d) { return (xScale(d[0]) + xScale.rangeBand() / 2) })
+			.attr('x', function (d) { return (xScale(d[0]) + xScale.rangeBand() / 2); })
 			.attr('y', function (d, i) { return innerH(); });
 		// update
 		bars
 			.attr('width', xScale.rangeBand() / 2)
-			.attr('x', function (d) { return (xScale(d[0]) + xScale.rangeBand() / 2) })
+			.attr('x', function (d) { return (xScale(d[0]) + xScale.rangeBand() / 2); })
 			.transition()
 			.delay(function (d, i) { return i / data.length * 1000; })
 			.duration(500)
