@@ -1,5 +1,5 @@
 /**
- * browsers.c -- functions for dealing with browsers
+ 6 browsers.c -- functions for dealing with browsers
  *    ______      ___
  *   / ____/___  /   | _____________  __________
  *  / / __/ __ \/ /| |/ ___/ ___/ _ \/ ___/ ___/
@@ -28,18 +28,24 @@
  * SOFTWARE.
  */
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stddef.h>
 
 #include "browsers.h"
 
+#include "error.h"
+#include "settings.h"
 #include "util.h"
 #include "xmalloc.h"
 
 /* ###NOTE: The size of the list is proportional to the run time,
  * which makes this pretty slow */
 
+static char ***browsers_hash = NULL;
+
 /* {"search string", "belongs to"} */
 static const char *browsers[][2] = {
   /* Game systems: most of them are based of major browsers,
@@ -109,6 +115,7 @@ static const char *browsers[][2] = {
   {"w3m", "Others"},
   {"MicroMessenger", "Others"},
   {"Apache", "Others"},
+  {"JOSM", "Others"},
 
   /* Feed-reader-as-a-service */
   {"AppleNewsBot", "Feeds"},
@@ -126,11 +133,11 @@ static const char *browsers[][2] = {
 
   /* Google crawlers (some based on Chrome,
    * therefore up on the list) */
-  {"Googlebot", "Crawlers"},
   {"AdsBot-Google", "Crawlers"},
   {"AppEngine-Google", "Crawlers"},
   {"Mediapartners-Google", "Crawlers"},
   {"Google", "Crawlers"},
+  {"WhatsApp", "Crawlers"},
 
   /* Based on Firefox */
   {"Camino", "Others"},
@@ -144,129 +151,38 @@ static const char *browsers[][2] = {
   {"YaBrowser", "Others"},
   {"Flock", "Others"},
   /* Chrome has to go before Safari */
+  {"HeadlessChrome", "Chrome"},
   {"Chrome", "Chrome"},
-
   {"CriOS", "Chrome"},
+
+  /* Crawlers/Bots (Possible Safari based) */
+  {"AppleBot", "Crawlers"},
+  {"Twitter", "Crawlers"},
+
   {"Safari", "Safari"},
 
   /* Crawlers/Bots */
-  {"bingbot", "Crawlers"},
-  {"msnbot", "Crawlers"},
-  {"YandexBot", "Crawlers"},
-  {"Baiduspider", "Crawlers"},
-  {"Ezooms", "Crawlers"},
-  {"Twitter", "Crawlers"},
-  {"Slurp", "Crawlers"},
-  {"Yahoo", "Crawlers"},
-  {"Slack", "Crawlers"},
-  {"AppleBot", "Crawlers"},
-  {"AhrefsBot", "Crawlers"},
-  {"Abonti", "Crawlers"},
-  {"MJ12bot", "Crawlers"},
-  {"SISTRIX", "Crawlers"},
-  {"facebook", "Crawlers"},
-  {"DotBot", "Crawlers"},
-  {"Speedy Spider", "Crawlers"},
-  {"Sosospider", "Crawlers"},
-  {"BPImageWalker", "Crawlers"},
   {"Sogou", "Crawlers"},
   {"Java", "Crawlers"},
   {"Jakarta Commons-HttpClient", "Crawlers"},
-  {"WBSearchBot", "Crawlers"},
-  {"SeznamBot", "Crawlers"},
-  {"DoCoMo", "Crawlers"},
-  {"TurnitinBot", "Crawlers"},
-  {"GSLFbot", "Crawlers"},
-  {"YodaoBot", "Crawlers"},
-  {"AddThis", "Crawlers"},
-  {"Purebot", "Crawlers"},
-  {"ia_archiver", "Crawlers"},
-  {"Wotbox", "Crawlers"},
-  {"CCBot", "Crawlers"},
-  {"findlinks", "Crawlers"},
-  {"Yeti", "Crawlers"},
-  {"ichiro", "Crawlers"},
-  {"Linguee Bot", "Crawlers"},
-  {"Gigabot", "Crawlers"},
-  {"BacklinkCrawler", "Crawlers"},
   {"netEstate", "Crawlers"},
-  {"distilator", "Crawlers"},
-  {"Aboundex", "Crawlers"},
-  {"UnwindFetchor", "Crawlers"},
-  {"SEOkicks-Robot", "Crawlers"},
-  {"psbot", "Crawlers"},
-  {"SBIder", "Crawlers"},
-  {"TestNutch", "Crawlers"},
-  {"DomainCrawler", "Crawlers"},
-  {"NextGenSearchBot", "Crawlers"},
-  {"SEOENGWorldBot", "Crawlers"},
   {"PiplBot", "Crawlers"},
   {"IstellaBot", "Crawlers"},
-  {"Cityreview", "Crawlers"},
   {"heritrix", "Crawlers"},
-  {"PagePeeker", "Crawlers"},
-  {"JS-Kit", "Crawlers"},
-  {"ScreenerBot", "Crawlers"},
   {"PagesInventory", "Crawlers"},
-  {"ShowyouBot", "Crawlers"},
-  {"SolomonoBot", "Crawlers"},
   {"rogerbot", "Crawlers"},
   {"fastbot", "Crawlers"},
-  {"Domnutch", "Crawlers"},
-  {"MaxPoint", "Crawlers"},
-  {"NCBot", "Crawlers"},
-  {"TosCrawler", "Crawlers"},
-  {"Updownerbot", "Crawlers"},
-  {"urlwatch", "Crawlers"},
-  {"OpenWebSpider", "Crawlers"},
-  {"WordPress", "Crawlers"},
   {"yacybot", "Crawlers"},
-  {"PEAR", "Crawlers"},
-  {"ZumBot", "Crawlers"},
-  {"YisouSpider", "Crawlers"},
-  {"W3C", "Crawlers"},
-  {"vcheck", "Crawlers"},
   {"PycURL", "Crawlers"},
   {"PHP", "Crawlers"},
-  {"PercolateCrawler", "Crawlers"},
-  {"NING", "Crawlers"},
-  {"gvfs", "Crawlers"},
-  {"Crowsnest", "Crawlers"},
-  {"CatchBot", "Crawlers"},
-  {"Combine", "Crawlers"},
-  {"A6-Indexer", "Crawlers"},
-  {"Altresium", "Crawlers"},
   {"AndroidDownloadManager", "Crawlers"},
-  {"Apache-HttpClient", "Crawlers"},
-  {"Comodo", "Crawlers"},
-  {"crawler4j", "Crawlers"},
-  {"Cricket", "Crawlers"},
-  {"EC2LinkFinder", "Crawlers"},
   {"Embedly", "Crawlers"},
-  {"envolk", "Crawlers"},
-  {"libwww-perl", "Crawlers"},
   {"ruby", "Crawlers"},
   {"Ruby", "Crawlers"},
   {"python", "Crawlers"},
   {"Python", "Crawlers"},
   {"LinkedIn", "Crawlers"},
-  {"GeoHasher", "Crawlers"},
-  {"HTMLParser", "Crawlers"},
-  {"MLBot", "Crawlers"},
-  {"Jaxified Bot", "Crawlers"},
-  {"LinkWalker", "Crawlers"},
   {"Microsoft-WebDAV", "Crawlers"},
-  {"nutch", "Crawlers"},
-  {"PostRank", "Crawlers"},
-  {"Image", "Crawlers"},
-  {"keybase-proofs", "Crawlers"},
-  {"SemrushBot", "Crawlers"},
-  {"CommonCrawler", "Crawlers"},
-  {"Mail.RU_Bot", "Crawlers"},
-  {"MegaIndex.ru", "Crawlers"},
-  {"XoviBot", "Crawlers"},
-  {"X-CAD-SE", "Crawlers"},
-  {"Safeassign", "Crawlers"},
 
   /* Podcast fetchers */
   {"Downcast", "Podcasts"},
@@ -287,14 +203,8 @@ static const char *browsers[][2] = {
   {"NetNewsWire", "Feeds"},
   {"RSSOwl", "Feeds"},
   {"Thunderbird", "Feeds"},
-  {"Vienna", "Feeds"},
-  {"Windows-RSS-Platform", "Feeds"},
-  {"newsbeuter", "Feeds"},
-  {"Wrangler", "Feeds"},
-  {"Fever", "Feeds"},
 
   {"Pingdom.com", "Uptime"},
-  {"UptimeRobot", "Uptime"},
   {"jetmon", "Uptime"},
   {"NodeUptime", "Uptime"},
   {"NewRelicPinger", "Uptime"},
@@ -305,6 +215,128 @@ static const char *browsers[][2] = {
   {"Mozilla", "Others"}
 };
 
+/* Free all browser entries from our array of key/value pairs. */
+void
+free_browsers_hash (void)
+{
+  size_t i;
+  int j;
+
+  for (i = 0; i < ARRAY_SIZE (browsers); ++i) {
+    free (browsers_hash[i][0]);
+    free (browsers_hash[i][1]);
+    free (browsers_hash[i]);
+  }
+  free (browsers_hash);
+
+
+  for (j = 0; j < conf.browsers_hash_idx; ++j) {
+    free (conf.user_browsers_hash[j][0]);
+    free (conf.user_browsers_hash[j][1]);
+    free (conf.user_browsers_hash[j]);
+  }
+  if (conf.browsers_file) {
+    free (conf.user_browsers_hash);
+  }
+}
+
+static int
+is_dup (char ***list, int len, const char *browser)
+{
+  int i;
+  /* check for dups */
+  for (i = 0; i < len; ++i) {
+    if (strcmp (browser, list[i][0]) == 0)
+      return 1;
+  }
+  return 0;
+}
+
+/* Set a browser/type pair into our multidimensional array of browsers.
+ *
+ * On duplicate functions returns void.
+ * Otherwise memory is mallo'd for our array entry. */
+static void
+set_browser (char ***list, int idx, const char *browser, const char *type)
+{
+  list[idx] = xcalloc (2, sizeof (char *));
+  list[idx][0] = xstrdup (browser);
+  list[idx][1] = xstrdup (type);
+}
+
+/* Parse the key/value pair from the browser list file. */
+static void
+parse_browser_token (char ***list, char *line, int n)
+{
+  char *val;
+  size_t idx = 0;
+
+  /* key */
+  idx = strcspn (line, "\t");
+  if (strlen (line) == idx)
+    FATAL ("Malformed browser name at line: %d", n);
+
+  line[idx] = '\0';
+
+  /* value */
+  val = line + (idx + 1);
+  idx = strspn (val, "\t");
+  if (strlen (val) == idx)
+    FATAL ("Malformed browser category at line: %d", n);
+  val = val + idx;
+  val = trim_str (val);
+
+  if (is_dup (list, conf.browsers_hash_idx, line)) {
+    LOG_INVALID (("Duplicate browser entry: %s", line));
+    return;
+  }
+
+  set_browser (list, conf.browsers_hash_idx, line, val);
+  conf.browsers_hash_idx++;
+}
+
+/* Parse our default array of browsers and put them on our hash including those
+ * from the custom parsed browsers file.
+ *
+ * On error functions returns void.
+ * Otherwise browser entries are put into the hash. */
+void
+parse_browsers_file (void)
+{
+  char line[MAX_LINE_BROWSERS + 1];
+  FILE *file;
+  int n = 0;
+  size_t i, len = ARRAY_SIZE (browsers);
+
+  browsers_hash = xmalloc (ARRAY_SIZE (browsers) * sizeof (char **));
+  /* load hash from the browser's array (default)  */
+  for (i = 0; i < len; ++i) {
+    set_browser (browsers_hash, i, browsers[i][0], browsers[i][1]);
+  }
+
+  if (!conf.browsers_file)
+    return;
+
+  /* could not open browsers file */
+  if ((file = fopen (conf.browsers_file, "r")) == NULL)
+    FATAL ("Unable to open browser's file: %s", strerror (errno));
+
+  conf.user_browsers_hash = xmalloc (MAX_CUSTOM_BROWSERS * sizeof (char **));
+  /* load hash from the user's given browsers file  */
+  while (fgets (line, sizeof line, file) != NULL) {
+    while (line[0] == ' ' || line[0] == '\t')
+      memmove (line, line + 1, strlen (line));
+    n++;
+
+    if (line[0] == '\n' || line[0] == '\r' || line[0] == '#')
+      continue;
+    if (conf.browsers_hash_idx >= MAX_CUSTOM_BROWSERS)
+      FATAL ("Maximum number of custom browsers has been reached");
+    parse_browser_token (conf.user_browsers_hash, line, n);
+  }
+  fclose (file);
+}
+
 /* Determine if the user-agent is a crawler.
  *
  * On error or is not a crawler, 0 is returned.
@@ -337,6 +369,118 @@ parse_opera (char *token)
   return val;
 }
 
+/* Given the original user agent string, and a partial crawler match, iterate
+ * back until the next delimiter is found and return occurrence.
+ *
+ * On error when attempting to extract crawler, NULL is returned.
+ * If a possible crawler string is matched, then possible bot is returned . */
+static char *
+parse_crawler (char *str, char *match, char *type)
+{
+  char *ptr = NULL;
+  int found = 0;
+
+  while (match != str) {
+    match--;
+    if (*match == ' ' || *match == '+') {
+      found = 1;
+      break;
+    }
+  }
+
+  /* same addr */
+  if (match == str)
+    return NULL;
+
+  /* account for the previous +|space */
+  if (found)
+    match++;
+
+  if ((ptr = strpbrk (match, "; ")))
+    *ptr = '\0';
+  /* empty string after parsing it */
+  if (*match == '\0')
+    return NULL;
+
+  xstrncpy (type, "Crawlers", BROWSER_TYPE_LEN);
+
+  return xstrdup (match);
+}
+
+/* If the following string matches are found within user agent, then it's
+ * highly likely it's a possible crawler.
+ * Note that this could certainly return false positives.
+ *
+ * If no occurrences are found, NULL is returned.
+ * If an occurrence is found, a pointer to the match is returned . */
+static char *
+check_http_crawler (const char *str)
+{
+  char *match = NULL;
+
+  /* e.g., compatible; bingbot/2.0; +http://www.bing.com/bingbot.htm */
+  if ((match = strstr (str, "; +http")))
+    return match;
+  /* compatible; UptimeRobot/2.0; http://www.uptimerobot.com/ */
+  if ((match = strstr (str, "; http")))
+    return match;
+  /* Slack-ImgProxy (+https://api.slack.com/robots) */
+  if ((match = strstr (str, " (+http")))
+    return match;
+  /*  TurnitinBot/3.0 (http://www.turnitin.com/robot/crawlerinfo.html) */
+  if ((match = strstr (str, " (http")))
+    return match;
+  /* w3c e.g., (compatible;+Googlebot/2.1;++http://www.google.com/bot.html) */
+  if ((match = strstr (str, ";++http")))
+    return match;
+  return NULL;
+}
+
+/* Parse the given user agent match and extract the browser string.
+ *
+ * If no match, the original match is returned.
+ * Otherwise the parsed browser is returned. */
+static char *
+parse_browser (char *match, char *type, int i, char ***hash)
+{
+  char *b = NULL, *ptr = NULL, *slh = NULL;
+  size_t cnt = 0, space = 0;
+
+  /* Check if there are spaces in the token string, that way strpbrk
+   * does not stop at the first space within the token string */
+  if ((cnt = count_matches (hash[i][0], ' ')) && (b = match)) {
+    while (space++ < cnt && (b = strchr (b, ' ')))
+      b++;
+  } else
+    b = match;
+
+  xstrncpy (type, hash[i][1], BROWSER_TYPE_LEN);
+  /* Internet Explorer 11 */
+  if (strstr (match, "rv:11") && strstr (match, "Trident/7.0")) {
+    return alloc_string ("MSIE/11.0");
+  }
+  /* Opera +15 uses OPR/# */
+  if (strstr (match, "OPR") != NULL && (slh = strrchr (match, '/'))) {
+    return parse_opera (slh);
+  }
+  /* Opera has the version number at the end */
+  if (strstr (match, "Opera") && (slh = strrchr (match, '/')) && match < slh) {
+    memmove (match + 5, slh, strlen (slh) + 1);
+  }
+  /* IE Old */
+  if (strstr (match, "MSIE") != NULL) {
+    if ((ptr = strpbrk (match, ";)-")) != NULL)
+      *ptr = '\0';
+    match = char_replace (match, ' ', '/');
+  }
+  /* all others */
+  else if ((ptr = strpbrk (b, ";) ")) != NULL) {
+    *ptr = '\0';
+  }
+
+  return alloc_string (match);
+}
+
 /* Given a user agent, determine the browser used.
  *
  * ###NOTE: The size of the list is proportional to the run time,
@@ -347,49 +491,31 @@ parse_opera (char *token)
 char *
 verify_browser (char *str, char *type)
 {
-  char *a, *b, *ptr, *slash;
-  size_t i;
+  char *match = NULL, *token = NULL;
+  int i = 0;
+  size_t j = 0;
 
   if (str == NULL || *str == '\0')
     return NULL;
 
-  for (i = 0; i < ARRAY_SIZE (browsers); i++) {
-    if ((a = strstr (str, browsers[i][0])) == NULL)
+  /* check user's list */
+  for (i = 0; i < conf.browsers_hash_idx; ++i) {
+    if ((match = strstr (str, conf.user_browsers_hash[i][0])) == NULL)
       continue;
-
-    /* check if there is a space char in the token string, that way strpbrk
-     * does not stop at the first space within the token string */
-    if ((strchr (browsers[i][0], ' ')) != NULL && (b = strchr (a, ' ')) != NULL)
-      b++;
-    else
-      b = a;
-
-    xstrncpy (type, browsers[i][1], BROWSER_TYPE_LEN);
-    /* Internet Explorer 11 */
-    if (strstr (a, "rv:11") && strstr (a, "Trident/7.0")) {
-      return alloc_string ("MSIE/11.0");
-    }
-    /* Opera +15 uses OPR/# */
-    if (strstr (a, "OPR") != NULL && (slash = strrchr (a, '/'))) {
-      return parse_opera (slash);
-    }
-    /* Opera has the version number at the end */
-    if (strstr (a, "Opera") && (slash = strrchr (a, '/')) && a < slash) {
-      memmove (a + 5, slash, strlen (slash) + 1);
-    }
-    /* IE Old */
-    if (strstr (a, "MSIE") != NULL) {
-      if ((ptr = strpbrk (a, ";)-")) != NULL)
-        *ptr = '\0';
-      a = char_replace (a, ' ', '/');
-    }
-    /* all others */
-    else if ((ptr = strpbrk (b, ";) ")) != NULL) {
-      *ptr = '\0';
-    }
-
-    return alloc_string (a);
+    return parse_browser (match, type, i, conf.user_browsers_hash);
   }
+
+  if ((match = check_http_crawler (str)) &&
+      (token = parse_crawler (str, match, type)))
+    return token;
+
+  /* fallback to default browser list */
+  for (j = 0; j < ARRAY_SIZE (browsers); ++j) {
+    if ((match = strstr (str, browsers_hash[j][0])) == NULL)
+      continue;
+    return parse_browser (match, type, j, browsers_hash);
+  }
+
   xstrncpy (type, "Unknown", BROWSER_TYPE_LEN);
 
   return alloc_string ("Unknown");
