@@ -39,7 +39,9 @@
 #include <unistd.h>
 
 #include "commons.h"
+
 #include "error.h"
+#include "labels.h"
 #include "settings.h"
 #include "util.h"
 #include "xmalloc.h"
@@ -81,9 +83,10 @@ get_max_choices (void)
   /* JSON - allow n amount of choices */
   if (find_output_type (&json, "json", 1) == 0 && conf.max_items > 0)
     max = conf.max_items;
-  /* HTML - takes priority on cases where multiple outputs were given */
-  if (find_output_type (&html, "html", 1) == 0 || conf.output_format_idx == 0 ||
-      !isatty (STDOUT_FILENO))
+  /* HTML - takes priority on cases where multiple outputs were given. Note that
+   * we check either for an .html extension or we assume not extension was passed
+   * via -o and therefore we are redirecting the output to a file. */
+  if (find_output_type (&html, "html", 1) == 0 || conf.output_format_idx == 0)
     max = conf.max_items > MAX_CHOICES ? MAX_CHOICES : conf.max_items;
 
   free (csv);
@@ -107,11 +110,11 @@ void
 display_storage (void)
 {
 #ifdef TCB_BTREE
-  fprintf (stdout, "Built using Tokyo Cabinet On-Disk B+ Tree.\n");
+  fprintf (stdout, "%s\n", BUILT_WITH_TCBTREE);
 #elif TCB_MEMHASH
-  fprintf (stdout, "Built using Tokyo Cabinet On-Memory Hash database.\n");
+  fprintf (stdout, "%s\n", BUILT_WITH_TCMEMHASH);
 #else
-  fprintf (stdout, "Built using the default On-Memory Hash database.\n");
+  fprintf (stdout, "%s\n", BUILT_WITH_DEFHASH);
 #endif
 }
 
@@ -122,8 +125,8 @@ display_default_config_file (void)
   char *path = get_config_file_path ();
 
   if (!path) {
-    fprintf (stdout, "No default config file found.\n");
-    fprintf (stdout, "You may specify one with `-p /path/goaccess.conf`\n");
+    fprintf (stdout, "%s\n", ERR_NODEF_CONF_FILE);
+    fprintf (stdout, "%s `-p /path/goaccess.conf`\n", ERR_NODEF_CONF_FILE_DESC);
   } else {
     fprintf (stdout, "%s\n", path);
     free (path);
@@ -135,8 +138,41 @@ void
 display_version (void)
 {
   fprintf (stdout, "GoAccess - %s.\n", GO_VERSION);
-  fprintf (stdout, "For more details visit: http://goaccess.io\n");
+  fprintf (stdout, "%s: http://goaccess.io\n", INFO_MORE_INFO);
   fprintf (stdout, "Copyright (C) 2009-2016 by Gerardo Orellana\n");
+  fprintf (stdout, "\nBuild configure arguments:\n");
+#ifdef DEBUG
+  fprintf (stdout, "  --enable-debug\n");
+#endif
+#ifdef HAVE_NCURSESW_NCURSES_H
+  fprintf (stdout, "  --enable-utf8\n");
+#endif
+#ifdef HAVE_LIBGEOIP
+  fprintf (stdout, "  --enable-geoip=legacy\n");
+#endif
+#ifdef HAVE_LIBMAXMINDDB
+  fprintf (stdout, "  --enable-geoip=mmdb\n");
+#endif
+#ifdef TCB_MEMHASH
+  fprintf (stdout, "  --enable-tcb=memhash\n");
+#endif
+#ifdef TCB_BTREE
+  fprintf (stdout, "  --enable-tcb=btree\n");
+#endif
+#if defined(TCB_MEMHASH) || defined(TCB_BTREE)
+#ifndef HAVE_ZLIB
+  fprintf (stdout, "  --disable-zlib\n");
+#endif
+#ifndef HAVE_BZ2
+  fprintf (stdout, "  --disable-bzip\n");
+#endif
+#endif
+#ifdef WITH_GETLINE
+  fprintf (stdout, "  --with-getline\n");
+#endif
+#ifdef HAVE_LIBSSL
+  fprintf (stdout, "  --with-openssl\n");
+#endif
 }
 
 /* Get the enumerated value given a string.
@@ -417,13 +453,18 @@ init_modules (void)
 /* Get the logs size.
  *
  * If log was piped (from stdin), 0 is returned.
- * On success, it adds up all log sizes and its value is returned. */
+ * On success, it adds up all log sizes and its value is returned.
+ * if --log-size was specified, it will be returned explicitly */
 intmax_t
 get_log_sizes (void)
 {
   int i;
   off_t size = 0;
 
+  /* --log-size */
+  if (conf.log_size > 0)
+    return (intmax_t) conf.log_size;
+
   for (i = 0; i < conf.filenames_idx; ++i) {
     if (conf.filenames[i][0] == '-' && conf.filenames[i][1] == '\0')
       size += 0;
