@@ -42,7 +42,7 @@
 #define __attribute__(x) /**/
 #endif
 #define GO_UNUSED __attribute__((unused))
-#define GO_VERSION 		"1.2"
+#define GO_VERSION 		"1.3"
 #define GO_WEBSITE 		"http://goaccess.io/"
 struct tm *now_tm;
 
@@ -77,6 +77,9 @@ struct tm *now_tm;
 /* maximum protocol string length */
 #define REQ_PROTO_LEN     9
 
+#define IGNORE_LEVEL_PANEL 1
+#define IGNORE_LEVEL_REQ 2
+
 /* Type of IP */
 typedef enum
 {
