@@ -42,6 +42,7 @@
 #include <unistd.h>
 
 #include "error.h"
+#include "labels.h"
 #include "parser.h"
 
 static FILE *access_log;
@@ -95,7 +96,8 @@ set_signal_data (void *p)
   log_data = p;
 }
 
-/* Open a access file whose name is specified in the given path. */
+/* Open the WebSocket access log file whose name is specified in the
+ * given path. */
 int
 access_log_open (const char *path)
 {
@@ -112,7 +114,7 @@ access_log_open (const char *path)
   return 0;
 }
 
-/* Close the access log file. */
+/* Close the WebSocket access log file. */
 void
 access_log_close (void)
 {
@@ -171,7 +173,7 @@ sigsegv_handler (int sig)
 #endif
 
   fprintf (fp, "==%d==\n", pid);
-  fprintf (fp, "==%d== Please report it by opening an issue on GitHub:\n", pid);
+  fprintf (fp, "==%d== %s:\n", pid, ERR_PLEASE_REPORT);
   fprintf (fp, "==%d== https://github.com/allinurl/goaccess/issues\n\n", pid);
   exit (EXIT_FAILURE);
 }
