@@ -54,7 +54,7 @@
            __TIME__);                                                         \
   fprintf (stderr, "Config file: %s\n", conf.iconfigfile ?: NO_CONFIG_FILE);  \
   fprintf (stderr, "\nFatal error has occurred");                             \
-  fprintf (stderr, "\nError occured at: %s - %s - %d\n", __FILE__,            \
+  fprintf (stderr, "\nError occurred at: %s - %s - %d\n", __FILE__,            \
            __FUNCTION__, __LINE__);                                           \
   fprintf (stderr, fmt, ##__VA_ARGS__);                                       \
   fprintf (stderr, "\n\n");                                                   \
