@@ -60,9 +60,10 @@
 #endif
 
 #ifdef HAVE_GEOLOCATION
-#include "geoip.h"
+#include "geoip1.h"
 #endif
 
+#include "browsers.h"
 #include "csv.h"
 #include "error.h"
 #include "gdashboard.h"
@@ -130,16 +131,10 @@ static GScroll gscroll = {
 };
 /* *INDENT-ON* */
 
-/* Free malloc'd data across the whole program */
+/* Free malloc'd holder */
 static void
-house_keeping (void)
+house_keeping_holder (void)
 {
-#ifdef TCB_MEMHASH
-  /* free malloc'd int values on the agent list */
-  if (conf.list_agents)
-    free_agent_list ();
-#endif
-
   /* REVERSE DNS THREAD */
   pthread_mutex_lock (&gdns_thread.mutex);
 
@@ -153,6 +148,19 @@ house_keeping (void)
   free_storage ();
 
   pthread_mutex_unlock (&gdns_thread.mutex);
+}
+
+/* Free malloc'd data across the whole program */
+static void
+house_keeping (void)
+{
+#ifdef TCB_MEMHASH
+  /* free malloc'd int values on the agent list */
+  if (conf.list_agents)
+    free_agent_list ();
+#endif
+
+  house_keeping_holder ();
 
   /* DASHBOARD */
   if (dash && !conf.output_stdout) {
@@ -179,6 +187,7 @@ house_keeping (void)
 
   /* CONFIGURATION */
   free_formats ();
+  free_browsers_hash ();
   if (conf.debug_log) {
     LOG_DEBUG (("Bye.\n"));
     dbg_log_close ();
@@ -196,6 +205,24 @@ house_keeping (void)
   free (gwsreader);
 }
 
+/* Open the pidfile whose name is specified in the given path and write
+ * the daemonized given pid. */
+static void
+write_pid_file (const char *path, pid_t pid)
+{
+  FILE *pidfile;
+
+  if (!path)
+    return;
+
+  if ((pidfile = fopen (path, "w"))) {
+    fprintf (pidfile, "%d", pid);
+    fclose (pidfile);
+  } else {
+    FATAL ("Unable to open the specified pid file. %s", strerror (errno));
+  }
+}
+
 /* Set GoAccess to run as a daemon */
 static void
 daemonize (void)
@@ -209,6 +236,7 @@ daemonize (void)
   if (pid < 0)
     exit (EXIT_FAILURE);
   if (pid > 0) {
+    write_pid_file (conf.pidfile, pid);
     printf ("Daemonized GoAccess: %d\n", pid);
     exit (EXIT_SUCCESS);
   }
@@ -338,9 +366,9 @@ render_screens (void)
   draw_header (stdscr, "", "%s", row - 1, 0, col, color_default);
 
   wattron (stdscr, color->attr | COLOR_PAIR (color->pair->idx));
-  mvaddstr (row - 1, 1, "[?] Help [Enter] Exp. Panel");
+  mvaddstr (row - 1, 1, T_HELP_ENTER);
   mvprintw (row - 1, 30, "%d - %s", chg, asctime (now_tm));
-  mvaddstr (row - 1, col - 21, "[Q]uit GoAccess");
+  mvaddstr (row - 1, col - 21, T_QUIT);
   mvprintw (row - 1, col - 5, "%s", GO_VERSION);
   wattroff (stdscr, color->attr | COLOR_PAIR (color->pair->idx));
 
@@ -379,8 +407,7 @@ disabled_panel_msg (GModule module)
   int row, col;
 
   getmaxyx (stdscr, row, col);
-  draw_header (stdscr, lbl, "'%s' panel is disabled", row - 1, 0, col,
-               color_error);
+  draw_header (stdscr, lbl, ERR_PANEL_DISABLED, row - 1, 0, col, color_error);
 }
 
 /* Set the current module/panel */
@@ -699,8 +726,8 @@ read_client (void *ptr_data)
   FD_ZERO (&rfds);
   FD_ZERO (&wfds);
 
-  /* open fifo for read */
-  if ((reader->fd = open_fifoout ()) == -1)
+  /* check we have a fifo for reading */
+  if (reader->fd == -1)
     return;
 
   pthread_mutex_lock (&reader->mutex);
@@ -1104,6 +1131,22 @@ set_general_stats (void)
 #endif
 }
 
+/* Store accumulated processing time
+ * Note: As we store with time_t second resolution,
+ * if elapsed time == 0, we will bump it to 1.
+ */
+#ifdef TCB_BTREE
+static void
+set_accumulated_time (void)
+{
+  if (conf.store_accumulated_time) {
+    time_t elapsed = end_proc - start_proc;
+    elapsed = (!elapsed) ? !elapsed : elapsed;
+    ht_insert_genstats_accumulated_time (elapsed);
+  }
+}
+#endif
+
 /* Execute the following calls right before we start the main
  * processing/parsing loop */
 static void
@@ -1158,6 +1201,10 @@ set_locale (void)
 {
   char *loc_ctype;
 
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
+
   loc_ctype = getenv ("LC_CTYPE");
   if (loc_ctype != NULL)
     setlocale (LC_CTYPE, loc_ctype);
@@ -1327,6 +1374,8 @@ initializer (void)
   /* setup to use the current locale */
   set_locale ();
 
+  parse_browsers_file ();
+
 #ifdef HAVE_GEOLOCATION
   init_geoip ();
 #endif
@@ -1355,6 +1404,12 @@ set_standard_output (void)
 
   /* Spawn WebSocket server threads */
   if (html && conf.real_time_html) {
+    /* open fifo for read */
+    if ((gwsreader->fd = open_fifoout ()) == -1) {
+      LOG (("Unable to open FIFO for read.\n"));
+      return;
+    }
+
     if (conf.daemonize)
       daemonize ();
     setup_ws_server (gwswriter, gwsreader);
@@ -1415,12 +1470,17 @@ main (int argc, char **argv)
 
   initializer ();
 
+  /* ignore outputting, process only */
+  if (conf.process_and_exit) {
+  }
   /* set stdout */
-  if (conf.output_stdout)
+  else if (conf.output_stdout) {
     set_standard_output ();
+  }
   /* set curses */
-  else
+  else {
     set_curses (&quit);
+  }
 
   /* no log/date/time format set */
   if (quit)
@@ -1445,14 +1505,20 @@ main (int argc, char **argv)
   end_spinner ();
   time (&end_proc);
 
-  /* stdout */
+  /* ignore outputting, process only */
   if (conf.process_and_exit) {
-    /* ignore outputting, process only */
-  } else if (conf.output_stdout)
+#ifdef TCB_BTREE
+    set_accumulated_time ();
+#endif
+  }
+  /* stdout */
+  else if (conf.output_stdout) {
     standard_output ();
+  }
   /* curses */
-  else
+  else {
     curses_output ();
+  }
 
   /* clean */
 clean:
