@@ -375,9 +375,9 @@ static void
 pskeyfval (GJSON * json, const char *key, float val, int sp, int last)
 {
   if (!last)
-    pjson (json, "%.*s\"%s\": %4.2f,%.*s", sp, TAB, key, val, nlines, NL);
+    pjson (json, "%.*s\"%s\": \"%4.2f\",%.*s", sp, TAB, key, val, nlines, NL);
   else
-    pjson (json, "%.*s\"%s\": %4.2f", sp, TAB, key, val);
+    pjson (json, "%.*s\"%s\": \"%4.2f\"", sp, TAB, key, val);
 }
 
 /* Write to a buffer the open block item object. */
@@ -524,7 +524,14 @@ poverall_invalid_reqs (GJSON * json, GLog * glog, int sp)
 static void
 poverall_processed_time (GJSON * json, int sp)
 {
-  pskeyu64val (json, OVERALL_GENTIME, end_proc - start_proc, sp, 0);
+  uint64_t elapsed_proc = end_proc - start_proc;
+
+#ifdef TCB_BTREE
+  if (conf.store_accumulated_time)
+    elapsed_proc = (uint64_t) ht_get_genstats ("accumulated_time");
+#endif
+
+  pskeyu64val (json, OVERALL_GENTIME, elapsed_proc, sp, 0);
 }
 
 /* Write to a buffer the total number of unique visitors under the
