@@ -50,9 +50,11 @@
 #include "options.h"
 
 #include "error.h"
+#include "labels.h"
 #include "util.h"
+#include "xmalloc.h"
 
-static char short_options[] = "f:e:p:o:l:H:M:"
+static char short_options[] = "f:e:p:o:l:H:M:S:b:"
 #ifdef HAVE_LIBGEOIP
   "g"
 #endif
@@ -61,6 +63,7 @@ static char short_options[] = "f:e:p:o:l:H:M:"
 /* *INDENT-OFF* */
 struct option long_opts[] = {
   {"agent-list"           , no_argument       , 0 , 'a' } ,
+  {"browsers-file"        , required_argument , 0 , 'b' } ,
   {"config-dialog"        , no_argument       , 0 , 'c' } ,
   {"config-file"          , required_argument , 0 , 'p' } ,
   {"debug-file"           , required_argument , 0 , 'l' } ,
@@ -73,6 +76,7 @@ struct option long_opts[] = {
   {"http-method"          , required_argument , 0 , 'M' } ,
   {"http-protocol"        , required_argument , 0 , 'H' } ,
   {"log-file"             , required_argument , 0 , 'f' } ,
+  {"log-size"             , required_argument , 0 , 'S' } ,
   {"no-query-string"      , no_argument       , 0 , 'q' } ,
   {"no-term-resolver"     , no_argument       , 0 , 'r' } ,
   {"output-format"        , required_argument , 0 , 'o' } ,
@@ -82,6 +86,7 @@ struct option long_opts[] = {
   {"with-output-resolver" , no_argument       , 0 , 'd' } ,
   {"444-as-404"           , no_argument       , 0 ,  0  } ,
   {"4xx-to-unique-count"  , no_argument       , 0 ,  0  } ,
+  {"anonymize-ip"         , no_argument       , 0 ,  0  } ,
   {"addr"                 , required_argument , 0 ,  0  } ,
   {"all-static-files"     , no_argument       , 0 ,  0  } ,
   {"color"                , required_argument , 0 ,  0  } ,
@@ -95,12 +100,14 @@ struct option long_opts[] = {
   {"enable-panel"         , required_argument , 0 ,  0  } ,
   {"fifo-in"              , required_argument , 0 ,  0  } ,
   {"fifo-out"             , required_argument , 0 ,  0  } ,
+  {"hide-referer"         , required_argument , 0 ,  0  } ,
   {"hour-spec"            , required_argument , 0 ,  0  } ,
   {"html-custom-css"      , required_argument , 0 ,  0  } ,
   {"html-custom-js"       , required_argument , 0 ,  0  } ,
   {"html-prefs"           , required_argument , 0 ,  0  } ,
   {"html-report-title"    , required_argument , 0 ,  0  } ,
   {"ignore-crawlers"      , no_argument       , 0 ,  0  } ,
+  {"ignore-statics"       , required_argument , 0 ,  0  } ,
   {"ignore-panel"         , required_argument , 0 ,  0  } ,
   {"ignore-referer"       , required_argument , 0 ,  0  } ,
   {"ignore-status"        , required_argument , 0 ,  0  } ,
@@ -113,11 +120,13 @@ struct option long_opts[] = {
   {"no-csv-summary"       , no_argument       , 0 ,  0  } ,
   {"no-global-config"     , no_argument       , 0 ,  0  } ,
   {"no-html-last-updated" , no_argument       , 0 ,  0  } ,
+  {"no-parsing-spinner"   , no_argument       , 0 ,  0  } ,
   {"no-progress"          , no_argument       , 0 ,  0  } ,
   {"no-tab-scroll"        , no_argument       , 0 ,  0  } ,
   {"num-tests"            , required_argument , 0 ,  0  } ,
   {"origin"               , required_argument , 0 ,  0  } ,
   {"output"               , required_argument , 0 ,  0  } ,
+  {"pid-file"             , required_argument , 0 ,  0  } ,
   {"port"                 , required_argument , 0 ,  0  } ,
   {"process-and-exit"     , no_argument       , 0 ,  0  } ,
   {"real-os"              , no_argument       , 0 ,  0  } ,
@@ -137,6 +146,7 @@ struct option long_opts[] = {
   {"geoip-city-data"      , required_argument , 0 ,  0  } ,
 #endif
 #ifdef TCB_BTREE
+  {"accumulated-time"     , no_argument       , 0 ,  0  } ,
   {"cache-lcnum"          , required_argument , 0 ,  0  } ,
   {"cache-ncnum"          , required_argument , 0 ,  0  } ,
   {"compression"          , required_argument , 0 ,  0  } ,
@@ -158,9 +168,10 @@ cmd_help (void)
   printf ("\nGoAccess - %s\n\n", GO_VERSION);
   printf (
   "Usage: "
-  "goaccess [filename] [ options ... ] [-c][-M][-H][-q][-d][...]\n"
-  "The following options can also be supplied to the command:\n\n"
+  "goaccess [filename] [ options ... ] [-c][-M][-H][-S][-q][-d][...]\n"
+  "%s:\n\n", INFO_HELP_FOLLOWING_OPTS);
 
+  printf (
   /* Log & Date Format Options */
   "Log & Date Format Options\n\n"
   "  --date-format=<dateformat>      - Specify log date format. e.g., %%d/%%b/%%Y\n"
@@ -186,9 +197,10 @@ cmd_help (void)
   "  --no-color                      - Disable colored output.\n"
   "  --no-column-names               - Don't write column names in term output.\n"
   "  --no-csv-summary                - Disable summary metrics on the CSV output.\n"
+  "  --no-html-last-updated          - Hide HTML last updated field.\n"
+  "  --no-parsing-spinner            - Disable progress metrics and parsing spinner.\n"
   "  --no-progress                   - Disable progress metrics.\n"
   "  --no-tab-scroll                 - Disable scrolling through panels on TAB.\n"
-  "  --no-html-last-updated          - Hide HTML last updated field.\n"
   "\n"
 
   /* Server Options */
@@ -199,6 +211,7 @@ cmd_help (void)
   "  --fifo-out=<path>               - Path to write named pipe (FIFO).\n"
   "  --origin=<addr>                 - Ensure clients send the specified origin header\n"
   "                                    upon the WebSocket handshake.\n"
+  "  --pid-file=<path>               - Write PID to a file when --daemonize is used.\n"
   "  --port=<port>                   - Specify the port to use.\n"
   "  --real-time-html                - Enable real-time HTML output.\n"
   "  --ssl-cert=<cert.crt>           - Path to TLS/SSL certificate.\n"
@@ -210,6 +223,7 @@ cmd_help (void)
   "File Options\n\n"
   "  -                               - The log file to parse is read from stdin.\n"
   "  -f --log-file=<filename>        - Path to input log file.\n"
+  "  -S --log-size=<number>          - Specify the log size, useful when piping in logs.\n"
   "  -l --debug-file=<filename>      - Send all debug messages to the specified\n"
   "                                    file.\n"
   "  -p --config-file=<filename>     - Custom configuration file.\n"
@@ -220,11 +234,12 @@ cmd_help (void)
   /* Parse Options */
   "Parse Options\n\n"
   "  -a --agent-list                 - Enable a list of user-agents by host.\n"
+  "  -b --browsers-file=<path>       - Use additional custom list of browsers.\n"
   "  -d --with-output-resolver       - Enable IP resolver on HTML|JSON output.\n"
   "  -e --exclude-ip=<IP>            - Exclude one or multiple IPv4/6. Allows IP\n"
   "                                    ranges e.g. 192.168.0.1-192.168.0.10\n"
   "  -H --http-protocol=<yes|no>     - Set/unset HTTP request protocol if found.\n"
-  "  -M --http-method=<yes|no>       - Set/unser HTTP request method if found.\n"
+  "  -M --http-method=<yes|no>       - Set/unset HTTP request method if found.\n"
   "  -o --output=file.html|json|csv  - Output either an HTML, JSON or a CSV file.\n"
   "  -q --no-query-string            - Ignore request's query string. Removing the\n"
   "                                    query string can greatly decrease memory\n"
@@ -233,18 +248,27 @@ cmd_help (void)
   "  --444-as-404                    - Treat non-standard status code 444 as 404.\n"
   "  --4xx-to-unique-count           - Add 4xx client errors to the unique visitors\n"
   "                                    count.\n"
+#ifdef TCB_BTREE
+  "  --accumulated-time              - Store accumulated time from parsing day-by-day logs.\n"
+#endif
+  "  --anonymize-ip                  - Anonymize IP addresses before outputting to report.\n"
   "  --all-static-files              - Include static files with a query string.\n"
   "  --crawlers-only                 - Parse and display only crawlers.\n"
   "  --date-spec=<date|hr>           - Date specificity. Possible values: `date`\n"
   "                                    (default), or `hr`.\n"
   "  --double-decode                 - Decode double-encoded values.\n"
   "  --enable-panel=<PANEL>          - Enable parsing/displaying the given panel.\n"
+  "  --hide-referer=<NEEDLE>         - Hide a referer but still count it. Wild cards\n"
+  "                                    are allowed. i.e., *.bing.com\n"
   "  --hour-spec=<hr|min>            - Hour specificity. Possible values: `hr`\n"
   "                                    (default), or `min` (tenth of a min).\n"
   "  --ignore-crawlers               - Ignore crawlers.\n"
   "  --ignore-panel=<PANEL>          - Ignore parsing/displaying the given panel.\n"
   "  --ignore-referer=<NEEDLE>       - Ignore a referer from being counted. Wild cards\n"
   "                                    are allowed. i.e., *.bing.com\n"
+  "  --ignore-statics=<req|panel>    - Ignore static requests.\n"
+  "                                    req => Ignore from valid requests.\n"
+  "                                    panel => Ignore from valid requests and panels.\n"
   "  --ignore-status=<CODE>          - Ignore parsing the given status code.\n"
   "  --num-tests=<number>            - Number of lines to test. >= 0 (10 default)\n"
   "  --process-and-exit              - Parse log and exit without outputting data.\n"
@@ -304,13 +328,14 @@ cmd_help (void)
   "                                    file when `-p` is not used.\n"
   "\n"
 
-  "Examples can be found by running `man goaccess`.\n\n"
-  "For more details visit: http://goaccess.io\n"
-  "GoAccess Copyright (C) 2009-2016 by Gerardo Orellana"
+  "%s `man goaccess`.\n\n"
+  "%s: http://goaccess.io\n"
+  "GoAccess Copyright (C) 2009-2017 by Gerardo Orellana"
   "\n\n"
 #ifdef TCB_BTREE
   , TC_DBPATH, TC_MMAP, TC_LCNUM, TC_NCNUM, TC_LMEMB, TC_NMEMB, TC_BNUM
 #endif
+  , INFO_HELP_EXAMPLES, INFO_MORE_INFO
   );
   exit (EXIT_FAILURE);
 }
@@ -398,6 +423,10 @@ parse_long_opt (const char *name, const char *oarg)
   if (!strcmp ("no-csv-summary", name))
     conf.no_csv_summary = 1;
 
+  /* no parsing spinner */
+  if (!strcmp ("no-parsing-spinner", name))
+    conf.no_parsing_spinner = 1;
+
   /* no progress */
   if (!strcmp ("no-progress", name))
     conf.no_progress = 1;
@@ -432,6 +461,10 @@ parse_long_opt (const char *name, const char *oarg)
   if (!strcmp ("origin", name))
     conf.origin = oarg;
 
+  /* PID file to write */
+  if (!strcmp ("pid-file", name))
+    conf.pidfile = oarg;
+
   /* port to use */
   if (!strcmp ("port", name)) {
     int port = strtol (oarg, NULL, 10);
@@ -480,6 +513,14 @@ parse_long_opt (const char *name, const char *oarg)
   if (!strcmp ("4xx-to-unique-count", name))
     conf.client_err_to_unique_count = 1;
 
+  /* anonymize ip */
+  if (!strcmp ("anonymize-ip", name))
+    conf.anonymize_ip = 1;
+
+  /* store accumulated time in tcb */
+  if (!strcmp ("accumulated-time", name))
+    conf.store_accumulated_time = 1;
+
   /* all static files */
   if (!strcmp ("all-static-files", name))
     conf.all_static_files = 1;
@@ -519,18 +560,33 @@ parse_long_opt (const char *name, const char *oarg)
     set_array_opt (oarg, conf.ignore_referers, &conf.ignore_referer_idx,
                    MAX_IGNORE_REF);
 
+  /* hide referer from report (e.g. within same site) */
+  if (!strcmp ("hide-referer", name))
+    set_array_opt (oarg, conf.hide_referers, &conf.hide_referer_idx,
+                   MAX_IGNORE_REF);
+
   /* ignore status code */
   if (!strcmp ("ignore-status", name))
     set_array_opt (oarg, conf.ignore_status, &conf.ignore_status_idx,
                    MAX_IGNORE_STATUS);
 
+  /* ignore static requests */
+  if (!strcmp ("ignore-statics", name)) {
+    if (!strcmp ("req", oarg))
+      conf.ignore_statics = IGNORE_LEVEL_REQ;
+    else if (!strcmp ("panel", oarg))
+      conf.ignore_statics = IGNORE_LEVEL_PANEL;
+    else
+      LOG_DEBUG (("Invalid statics ignore option."));
+  }
+
   /* number of line tests */
   if (!strcmp ("num-tests", name)) {
     char *sEnd;
     int tests = strtol (oarg, &sEnd, 10);
     if (oarg == sEnd || *sEnd != '\0' || errno == ERANGE)
       return;
-    conf.num_tests = tests;
+    conf.num_tests = tests >= 0 ? tests : 0;
   }
 
   /* process and exit */
@@ -546,9 +602,8 @@ parse_long_opt (const char *name, const char *oarg)
     set_array_opt (oarg, conf.sort_panels, &conf.sort_panel_idx, TOTAL_MODULES);
 
   /* static file */
-  if (!strcmp ("static-file", name)) {
-    if (conf.static_file_max_len < strlen (oarg) &&
-        conf.static_file_idx < MAX_EXTENSIONS)
+  if (!strcmp ("static-file", name) && conf.static_file_idx < MAX_EXTENSIONS) {
+    if (conf.static_file_max_len < strlen (oarg))
       conf.static_file_max_len = strlen (oarg);
     set_array_opt (oarg, conf.static_files, &conf.static_file_idx,
                    MAX_EXTENSIONS);
@@ -631,7 +686,7 @@ verify_global_config (int argc, char **argv)
 
     switch (o) {
     case 'p':
-      conf.iconfigfile = optarg;
+      conf.iconfigfile = xstrdup (optarg);
       break;
     case 0:
       if (!strcmp ("no-global-config", long_opts[idx].name))
@@ -646,10 +701,17 @@ verify_global_config (int argc, char **argv)
   optind = 1;
 }
 
-/* Attempt to add - to the array of filenames if it has been added it yet. */
+/* Attempt to add - to the array of filenames if it hasn't been added it yet. */
 void
 add_dash_filename (void)
 {
+  int i;
+  // pre-scan for '-' and don't add if already exists: github.com/allinurl/goaccess/issues/907
+  for (i = 0; i < conf.filenames_idx; ++i) {
+    if (conf.filenames[i][0] == '-' && conf.filenames[i][1] == '\0')
+      return;
+  }
+
   if (conf.filenames_idx < MAX_FILENAMES && !conf.read_stdin) {
     conf.read_stdin = 1;
     conf.filenames[conf.filenames_idx++] = "-";
@@ -674,6 +736,13 @@ read_option_args (int argc, char **argv)
       if (conf.filenames_idx < MAX_FILENAMES)
         conf.filenames[conf.filenames_idx++] = optarg;
       break;
+    case 'S':
+      if (strchr (optarg, '-')) {
+        printf ("[ERROR] log-size must be a positive integer\n");
+        exit (EXIT_FAILURE);
+      }
+      conf.log_size = (uint64_t) atoll (optarg);
+      break;
     case 'p':
       /* ignore it */
       break;
@@ -685,10 +754,15 @@ read_option_args (int argc, char **argv)
     case 'e':
       if (conf.ignore_ip_idx < MAX_IGNORE_IPS)
         conf.ignore_ips[conf.ignore_ip_idx++] = optarg;
+      else
+        LOG_DEBUG (("Max num of (%d) IPs to ignore reached.", MAX_IGNORE_IPS));
       break;
     case 'a':
       conf.list_agents = 1;
       break;
+    case 'b':
+      conf.browsers_file = optarg;
+      break;
     case 'c':
       conf.load_conf_dlg = 1;
       break;
@@ -699,6 +773,11 @@ read_option_args (int argc, char **argv)
       conf.ignore_qstr = 1;
       break;
     case 'o':
+      if (!valid_output_type (optarg)) {
+        printf
+          ("[ERROR] Invalid filename extension used, must be any of .csv, .json, or .html\n");
+        exit (EXIT_FAILURE);
+      }
       if (conf.output_format_idx < MAX_OUTFORMATS)
         conf.output_formats[conf.output_format_idx++] = optarg;
       break;
