@@ -63,7 +63,7 @@
 #endif
 
 #ifdef HAVE_GEOLOCATION
-#include "geoip.h"
+#include "geoip1.h"
 #endif
 
 #include "parser.h"
@@ -570,7 +570,10 @@ extract_keyphrase (char *ref, char **keyphrase)
 
   if (!(strstr (ref, "http://www.google.")) &&
       !(strstr (ref, "http://webcache.googleusercontent.com/")) &&
-      !(strstr (ref, "http://translate.googleusercontent.com/")))
+      !(strstr (ref, "http://translate.googleusercontent.com/")) &&
+      !(strstr (ref, "https://www.google.")) &&
+      !(strstr (ref, "https://webcache.googleusercontent.com/")) &&
+      !(strstr (ref, "https://translate.googleusercontent.com/")))
     return 1;
 
   /* webcache.googleusercontent */
@@ -676,9 +679,9 @@ clean:
  * On error, or if not static, 0 is returned.
  * On success, the 1 is returned. */
 static int
-verify_static_content (char *req)
+verify_static_content (const char *req)
 {
-  char *nul = req + strlen (req);
+  const char *nul = req + strlen (req);
   const char *ext = NULL, *pch = NULL;
   int elen = 0, i;
 
@@ -713,66 +716,39 @@ verify_static_content (char *req)
 static const char *
 extract_method (const char *token)
 {
-  const char *lookfor = NULL;
+  const char *methods[] = {
+    "OPTIONS", "GET", "HEAD", "POST", "PUT",
+    "DELETE", "TRACE", "CONNECT", "PATCH", "options",
+    "get", "head", "post", "put", "delete",
+    "trace", "connect", "patch",
+    /* WebDAV */
+    "PROPFIND", "PROPPATCH", "MKCOL", "COPY", "MOVE",
+    "LOCK", "UNLOCK", "VERSION-CONTROL", "REPORT", "CHECKOUT",
+    "CHECKIN", "UNCHECKOUT", "MKWORKSPACE", "UPDATE", "LABEL",
+    "MERGE", "BASELINE-CONTROL", "MKACTIVITY", "ORDERPATCH", "propfind",
+    "propwatch", "mkcol", "copy", "move", "lock",
+    "unlock", "version-control", "report", "checkout", "checkin",
+    "uncheckout", "mkworkspace", "update", "label", "merge",
+    "baseline-control", "mkactivity", "orderpatch"
+  };
 
-  if ((lookfor = "OPTIONS", !strncmp (token, lookfor, 7)) ||
-      (lookfor = "GET", !strncmp (token, lookfor, 3)) ||
-      (lookfor = "HEAD", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "POST", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "PUT", !strncmp (token, lookfor, 3)) ||
-      (lookfor = "DELETE", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "TRACE", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "CONNECT", !strncmp (token, lookfor, 7)) ||
-      (lookfor = "PATCH", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "options", !strncmp (token, lookfor, 7)) ||
-      (lookfor = "get", !strncmp (token, lookfor, 3)) ||
-      (lookfor = "head", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "post", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "put", !strncmp (token, lookfor, 3)) ||
-      (lookfor = "delete", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "trace", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "connect", !strncmp (token, lookfor, 7)) ||
-      (lookfor = "patch", !strncmp (token, lookfor, 5)) ||
-      /* WebDAV */
-      (lookfor = "PROPFIND", !strncmp (token, lookfor, 8)) ||
-      (lookfor = "PROPPATCH", !strncmp (token, lookfor, 9)) ||
-      (lookfor = "MKCOL", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "COPY", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "MOVE", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "LOCK", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "UNLOCK", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "VERSION-CONTROL", !strncmp (token, lookfor, 15)) ||
-      (lookfor = "REPORT", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "CHECKOUT", !strncmp (token, lookfor, 8)) ||
-      (lookfor = "CHECKIN", !strncmp (token, lookfor, 7)) ||
-      (lookfor = "UNCHECKOUT", !strncmp (token, lookfor, 10)) ||
-      (lookfor = "MKWORKSPACE", !strncmp (token, lookfor, 11)) ||
-      (lookfor = "UPDATE", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "LABEL", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "MERGE", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "BASELINE-CONTROL", !strncmp (token, lookfor, 16)) ||
-      (lookfor = "MKACTIVITY", !strncmp (token, lookfor, 10)) ||
-      (lookfor = "ORDERPATCH", !strncmp (token, lookfor, 10)) ||
-      (lookfor = "propfind", !strncmp (token, lookfor, 8)) ||
-      (lookfor = "propwatch", !strncmp (token, lookfor, 9)) ||
-      (lookfor = "mkcol", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "copy", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "move", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "lock", !strncmp (token, lookfor, 4)) ||
-      (lookfor = "unlock", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "version-control", !strncmp (token, lookfor, 15)) ||
-      (lookfor = "report", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "checkout", !strncmp (token, lookfor, 8)) ||
-      (lookfor = "checkin", !strncmp (token, lookfor, 7)) ||
-      (lookfor = "uncheckout", !strncmp (token, lookfor, 10)) ||
-      (lookfor = "mkworkspace", !strncmp (token, lookfor, 11)) ||
-      (lookfor = "update", !strncmp (token, lookfor, 6)) ||
-      (lookfor = "label", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "merge", !strncmp (token, lookfor, 5)) ||
-      (lookfor = "baseline-control", !strncmp (token, lookfor, 16)) ||
-      (lookfor = "mkactivity", !strncmp (token, lookfor, 10)) ||
-      (lookfor = "orderpatch", !strncmp (token, lookfor, 10)))
-    return lookfor;
+  const int methods_count = sizeof (methods) / sizeof (*methods);
+
+  int i;
+  /* Length of every string in list */
+  static int list_length[sizeof (methods) / sizeof (*methods)] = { -1 };
+  /* Only calculate length on first time */
+  if (list_length[0] == -1) {
+    for (i = 0; i < methods_count; i++) {
+      list_length[i] = strlen (methods[i]);
+    }
+  }
+
+  for (i = 0; i < methods_count; i++) {
+    if (strncmp (token, methods[i], list_length[i]) == 0) {
+      return methods[i];
+    }
+  }
   return NULL;
 }
 
@@ -793,14 +769,16 @@ contains_usecs (void)
  *
  * If not valid, 1 is returned.
  * If valid, 0 is returned. */
-static int
-invalid_protocol (const char *token)
+static const char *
+extract_protocol (const char *token)
 {
   const char *lookfor;
 
-  return !((lookfor = "HTTP/1.0", !strncmp (token, lookfor, 8)) ||
-           (lookfor = "HTTP/1.1", !strncmp (token, lookfor, 8)) ||
-           (lookfor = "HTTP/2", !strncmp (token, lookfor, 6)));
+  if ((lookfor = "HTTP/1.0", !strncmp (token, lookfor, 8)) ||
+      (lookfor = "HTTP/1.1", !strncmp (token, lookfor, 8)) ||
+      (lookfor = "HTTP/2", !strncmp (token, lookfor, 6)))
+    return lookfor;
+  return NULL;
 }
 
 /* Parse a request containing the method and protocol.
@@ -811,8 +789,8 @@ invalid_protocol (const char *token)
 static char *
 parse_req (char *line, char **method, char **protocol)
 {
-  char *req = NULL, *request = NULL, *proto = NULL, *dreq = NULL;
-  const char *meth;
+  char *req = NULL, *request = NULL, *dreq = NULL, *ptr = NULL;
+  const char *meth, *proto;
   ptrdiff_t rlen;
 
   meth = extract_method (line);
@@ -824,14 +802,11 @@ parse_req (char *line, char **method, char **protocol)
   /* method found, attempt to parse request */
   else {
     req = line + strlen (meth);
-    if ((proto = strstr (line, " HTTP/1.0")) == NULL &&
-        (proto = strstr (line, " HTTP/1.1")) == NULL &&
-        (proto = strstr (line, " HTTP/2")) == NULL) {
+    if (!(ptr = strrchr (req, ' ')) || !(proto = extract_protocol (++ptr)))
       return alloc_string ("-");
-    }
 
     req++;
-    if ((rlen = proto - req) <= 0)
+    if ((rlen = ptr - req) <= 0)
       return alloc_string ("-");
 
     request = xmalloc (rlen + 1);
@@ -842,15 +817,18 @@ parse_req (char *line, char **method, char **protocol)
       (*method) = strtoupper (xstrdup (meth));
 
     if (conf.append_protocol)
-      (*protocol) = strtoupper (xstrdup (++proto));
+      (*protocol) = strtoupper (xstrdup (proto));
   }
 
-  if ((dreq = decode_url (request)) && dreq != '\0') {
-    free (request);
-    return dreq;
+  if (!(dreq = decode_url (request)))
+    return request;
+  else if (*dreq == '\0') {
+    free (dreq);
+    return request;
   }
 
-  return request;
+  free (request);
+  return dreq;
 }
 
 /* Extract the next delimiter given a log format and copy the
@@ -871,7 +849,7 @@ get_delim (char *dest, const char *p)
   /* add the first delim */
   dest[0] = *(p + 1);
   /* check if there's another possible delim */
-  if (p[2] == '|' && p[3] != '\0') {
+  if (p[2] == '|' && p[3] != '%' && p[3] != '\0') {
     dest[1] = *(p + 3);
     return 1;
   }
@@ -959,6 +937,23 @@ set_date (char **fdate, struct tm tm)
   return 0;
 }
 
+/* Format the broken-down time tm to a numeric time format.
+ *
+ * On error, or unable to format the given tm, 1 is returned.
+ * On success, a malloc'd format is returned. */
+static int
+set_time (char **ftime, struct tm tm)
+{
+  char buf[TIME_LEN] = "";
+
+  memset (buf, 0, sizeof (buf));
+  if (strftime (buf, TIME_LEN, "%H:%M:%S", &tm) <= 0)
+    return 1;
+  *ftime = xstrdup (buf);
+
+  return 0;
+}
+
 /* Determine the parsing specifier error and construct a message out
  * of it.
  *
@@ -1042,12 +1037,12 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
-    if (str_to_time (tkn, tfmt, &tm) != 0) {
+    if (str_to_time (tkn, tfmt, &tm) != 0 || set_time (&logitem->time, tm) != 0) {
       spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
-    logitem->time = tkn;
+    free (tkn);
     break;
     /* date/time as decimal, i.e., timestamps, ms/us  */
   case 'x':
@@ -1056,12 +1051,12 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
-    if (str_to_time (tkn, tfmt, &tm) != 0 || set_date (&logitem->date, tm) != 0) {
+    if (str_to_time (tkn, tfmt, &tm) != 0 || set_date (&logitem->date, tm) != 0
+        || set_time (&logitem->time, tm) != 0) {
       spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
-    logitem->time = tkn;
     break;
     /* Virtual Host */
   case 'v':
@@ -1146,7 +1141,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, SPEC_TOKN_NUL, *p, NULL);
 
-    if (invalid_protocol (tkn)) {
+    if (!extract_protocol (tkn)) {
       spec_err (logitem, SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
@@ -1207,8 +1202,17 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (strcmp (tkn, "-") != 0) {
       extract_keyphrase (tkn, &logitem->keyphrase);
       extract_referer_site (tkn, logitem->site);
+
+      /* hide referrers from report */
+      if (hide_referer (logitem->site)) {
+        logitem->site[0] = '\0';
+        free (tkn);
+      } else
+        logitem->ref = tkn;
+      break;
     }
     logitem->ref = tkn;
+
     break;
     /* user agent */
   case 'u':
@@ -1219,7 +1223,7 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
     if (tkn != NULL && *tkn != '\0') {
       /* Make sure the user agent is decoded (i.e.: CloudFront)
        * and replace all '+' with ' ' (i.e.: w3c) */
-      logitem->agent = char_replace (decode_url (tkn), '+', ' ');
+      logitem->agent = decode_url (tkn);
       free (tkn);
       break;
     } else if (tkn != NULL && *tkn == '\0') {
@@ -1308,19 +1312,34 @@ parse_specifier (GLogItem * logitem, char **str, const char *p, const char *end)
 static char *
 extract_braces (char **p)
 {
-  const char *b1 = NULL;
-  size_t len = 0;
-  char *ret = NULL;
+  char *b1 = NULL, *b2 = NULL, *ret = NULL, *s = *p;
+  int esc = 0;
+  ptrdiff_t len = 0;
 
-  if (!(b1 = strchr (*p, '{')) || !(*p = strchr (*p + 1, '}')))
+  /* iterate over the log format */
+  for (; *s; s++) {
+    if (*s == '\\') {
+      esc = 1;
+    } else if (*s == '{' && !esc) {
+      b1 = s;
+    } else if (*s == '}' && !esc) {
+      b2 = s;
+      break;
+    } else {
+      esc = 0;
+    }
+  }
+
+  if ((!b1) || (!b2))
+    return NULL;
+  if ((len = b2 - (b1 + 1)) <= 0)
     return NULL;
 
   /* Found braces, extract 'reject' character set. */
-  len = *p - (b1 + 1);
   ret = xmalloc (len + 1);
   memcpy (ret, b1 + 1, len);
   ret[len] = '\0';
-  (*p)++;
+  (*p) = b2 + 1;
 
   return ret;
 }
@@ -1386,7 +1405,7 @@ special_specifier (GLogItem * logitem, char **str, char **p)
     if (logitem->host)
       return spec_err (logitem, SPEC_TOKN_SET, **p, NULL);
     if (find_xff_host (logitem, str, p))
-      return 1;
+      return spec_err (logitem, SPEC_TOKN_NUL, 'h', NULL);
     break;
   }
 
@@ -1417,7 +1436,7 @@ parse_format (GLogItem * logitem, char *str)
       perc++;
       continue;
     }
-    if (*p == '~') {
+    if (*p == '~' && perc == 0) {
       tilde++;
       continue;
     }
@@ -1517,12 +1536,12 @@ output_logerrors (GLog * glog)
   if (!glog->log_erridx)
     return;
 
-  fprintf (stderr, "Parsed %d lines ", glog->log_erridx);
-  fprintf (stderr, "producing the following errors:\n\n");
+  fprintf (stderr, ERR_PARSED_NLINES, glog->log_erridx);
+  fprintf (stderr, "%s:\n\n", ERR_PARSED_NLINES_DESC);
 
   for (i = 0; i < glog->log_erridx; ++i)
     fprintf (stderr, "%s\n", glog->errors[i]);
-  fprintf (stderr, "\n%s\n", ERR_HEADER);
+  fprintf (stderr, "\n%s\n", ERR_FORMAT_HEADER);
 }
 
 /* Free all log errors stored during parsing. */
@@ -1662,6 +1681,16 @@ handle_crawler (const char *agent)
   return (conf.ignore_crawlers && bot) || (conf.crawlers_only && !bot) ? 0 : 1;
 }
 
+/* A wrapper function to determine if the request is static.
+ *
+ * If the request is not static, 0 is returned.
+ * If the request is static, 1 is returned. */
+static int
+is_static (const char *req)
+{
+  return verify_static_content (req);
+}
+
 /* Determine if the request of the given status code needs to be
  * ignored.
  *
@@ -1678,39 +1707,18 @@ ignore_status_code (const char *status)
   return 0;
 }
 
-/* A wrapper function to determine if a log line needs to be ignored.
- *
- * If the request line is not ignored, 0 is returned.
- * If the request line is ignored, 1 is returned. */
+/* Determine if static file request should be ignored
+    *
+    * If the request line is not ignored, 0 is returned.
+    * If the request line is ignored, 1 is returned. */
 static int
-ignore_line (GLog * glog, GLogItem * logitem)
+ignore_static (const char *req)
 {
-  if (excluded_ip (glog, logitem) == 0)
+  if (conf.ignore_statics && is_static (req))
     return 1;
-  if (handle_crawler (logitem->agent) == 0)
-    return 1;
-  if (ignore_referer (logitem->site))
-    return 1;
-  if (ignore_status_code (logitem->status))
-    return 1;
-
-  /* check if we need to remove the request's query string */
-  if (conf.ignore_qstr)
-    strip_qstring (logitem->req);
-
   return 0;
 }
 
-/* A wrapper function to determine if the request is static.
- *
- * If the request is not static, 0 is returned.
- * If the request is static, 1 is returned. */
-static int
-is_static (GLogItem * logitem)
-{
-  return verify_static_content (logitem->req);
-}
-
 /* Determine if the request status code is a 404.
  *
  * If the request is not a 404, 0 is returned.
@@ -1728,6 +1736,32 @@ is_404 (GLogItem * logitem)
   return 0;
 }
 
+/* A wrapper function to determine if a log line needs to be ignored.
+ *
+ * If the request line is not ignored, 0 is returned.
+ * If the request line is ignored, IGNORE_LEVEL_PANEL is returned.
+ * If the request line is only not counted as valid, IGNORE_LEVEL_REQ is returned. */
+static int
+ignore_line (GLog * glog, GLogItem * logitem)
+{
+  if (excluded_ip (glog, logitem) == 0)
+    return IGNORE_LEVEL_PANEL;
+  if (handle_crawler (logitem->agent) == 0)
+    return IGNORE_LEVEL_PANEL;
+  if (ignore_referer (logitem->site))
+    return IGNORE_LEVEL_PANEL;
+  if (ignore_status_code (logitem->status))
+    return IGNORE_LEVEL_PANEL;
+  if (ignore_static (logitem->req))
+    return conf.ignore_statics; // IGNORE_LEVEL_PANEL or IGNORE_LEVEL_REQ
+
+  /* check if we need to remove the request's query string */
+  if (conf.ignore_qstr)
+    strip_qstring (logitem->req);
+
+  return 0;
+}
+
 /* A wrapper function to insert a keymap string key.
  *
  * If the given key exists, its value is returned.
@@ -1848,7 +1882,7 @@ insert_agent (int data_nkey, int agent_nkey, GModule module)
 static char *
 get_uniq_visitor_key (GLogItem * logitem)
 {
-  char *ua, *key;
+  char *ua = NULL, *key = NULL;
   size_t s1, s2, s3;
 
   ua = deblank (xstrdup (logitem->agent));
@@ -1858,7 +1892,7 @@ get_uniq_visitor_key (GLogItem * logitem)
   s3 = strlen (ua);
 
   /* includes terminating null */
-  key = xmalloc (s1 + s2 + s3 + 3);
+  key = xcalloc (s1 + s2 + s3 + 3, sizeof (char));
 
   memcpy (key, logitem->host, s1);
 
@@ -1903,7 +1937,7 @@ gen_unique_req_key (GLogItem * logitem)
   }
 
   /* includes terminating null */
-  key = xmalloc (s1 + s2 + s3 + nul);
+  key = xcalloc (s1 + s2 + s3 + nul, sizeof (char));
   /* append request */
   memcpy (key, logitem->req, s1);
 
@@ -1983,7 +2017,7 @@ set_spec_visitor_key (char **fdate, const char *ftime)
 }
 
 /* Generate a unique key for the visitors panel from the given logitem
- * structure and assign it to out key data structure.
+ * structure and assign it to the output key data structure.
  *
  * On error, or if no date is found, 1 is returned.
  * On success, the date key is assigned to our key data structure.
@@ -2326,7 +2360,7 @@ gen_visit_time_key (GKeyData * kdata, GLogItem * logitem)
       0)
     return 1;
 
-  if (hour == '\0')
+  if (*hour == '\0')
     return 1;
 
   if ((hmark = strchr (hour, ':')))
@@ -2480,6 +2514,7 @@ pre_process_log (GLog * glog, char *line, int dry_run)
 {
   GLogItem *logitem;
   int ret = 0;
+  int ignorelevel = 0;
 
   /* soft ignore these lines */
   if (valid_line (line))
@@ -2502,20 +2537,24 @@ pre_process_log (GLog * glog, char *line, int dry_run)
   if (dry_run)
     goto cleanup;
 
+  ignorelevel = ignore_line (glog, logitem);
   /* ignore line */
-  if (ignore_line (glog, logitem))
+  if (ignorelevel == IGNORE_LEVEL_PANEL)
     goto cleanup;
 
   if (is_404 (logitem))
     logitem->is_404 = 1;
-  else if (is_static (logitem))
+  else if (is_static (logitem->req))
     logitem->is_static = 1;
 
   logitem->uniq_key = get_uniq_visitor_key (logitem);
 
   inc_resp_size (glog, logitem->resp_size);
   process_log (logitem);
-  count_valid (glog);
+
+  /* don't ignore line but neither count as valid */
+  if (ignorelevel != IGNORE_LEVEL_REQ)
+    count_valid (glog);
 
 cleanup:
   free_glog (logitem);
@@ -2523,10 +2562,16 @@ cleanup:
   return ret;
 }
 
+/* Entry point to process the given live from the log.
+ *
+ * On error, 1 is returned.
+ * On success or soft ignores, 0 is returned. */
 static int
 read_line (GLog * glog, char *line, int *test, int *cnt, int dry_run)
 {
   int ret = 0;
+  int tests = conf.num_tests;
+
   /* start processing log line */
   if ((ret = pre_process_log (glog, line, dry_run)) == 0 && *test)
     *test = 0;
@@ -2534,9 +2579,14 @@ read_line (GLog * glog, char *line, int *test, int *cnt, int dry_run)
   /* soft ignores */
   if (ret == -1)
     return 0;
+
+  /* glog->processed can actually be less than conf.num_tests, so we make sure
+   * (cnt) compares to the right number */
+  tests = MIN (conf.num_tests, glog->processed);
+
   /* reached num of lines to test and no valid records were found, log
    * format is likely not matching */
-  if (conf.num_tests >= 0 && ++(*cnt) == conf.num_tests && *test) {
+  if (conf.num_tests && ++(*cnt) == tests && *test) {
     uncount_processed (glog);
     uncount_invalid (glog);
     return 1;
@@ -2556,7 +2606,16 @@ fgetline (FILE * fp)
   char *line = NULL, *tmp = NULL;
   size_t linelen = 0, len = 0;
 
-  while (fgets (buf, sizeof (buf), fp)) {
+  while (1) {
+    if (!fgets (buf, sizeof (buf), fp)) {
+      if (conf.process_and_exit && errno == EAGAIN) {
+        nanosleep ((const struct timespec[]) { {
+                   0, 100000000L}}, NULL);
+        continue;
+      } else
+        break;
+    }
+
     len = strlen (buf);
 
     /* overflow check */
@@ -2607,7 +2666,7 @@ read_lines (FILE * fp, GLog ** glog, int dry_run)
   if (!line && (errno == EAGAIN || errno == EWOULDBLOCK) && test)
     return 0;
 
-  return test || ret;
+  return (line && test) || ret;
 
 out:
   free (line);
@@ -2642,7 +2701,7 @@ read_lines (FILE * fp, GLog ** glog, int dry_run)
   if (!s && (errno == EAGAIN || errno == EWOULDBLOCK) && test)
     return 0;
 
-  return test || ret;
+  return (s && test) || ret;
 }
 #endif
 
