@@ -40,6 +40,7 @@
 #include "settings.h"
 
 #include "error.h"
+#include "labels.h"
 #include "util.h"
 #include "xmalloc.h"
 
@@ -129,17 +130,16 @@ get_config_file_path (void)
     return rpath;
   }
 
-  /* otherwise, fallback to global config file, or
-   * attempt to use the user's config file */
-  if (conf.load_global_config) {
-    upath = get_global_config ();
-    rpath = realpath (upath, NULL);
-    if (upath) {
-      free (upath);
-    }
+  /* attempt to use the user's config file */
+  upath = get_home ();
+  rpath = realpath (upath, NULL);
+  if (upath) {
+    free (upath);
   }
-  if (rpath == NULL) {
-    upath = get_home ();
+
+  /* otherwise, fallback to global config file */
+  if (rpath == NULL && conf.load_global_config) {
+    upath = get_global_config ();
     rpath = realpath (upath, NULL);
     if (upath) {
       free (upath);
@@ -167,12 +167,8 @@ set_default_static_files (void)
     ".pdf",
     ".txt",
     ".csv",
-    ".zip",
-    ".mp3",
-    ".mp4",
     ".mpeg",
     ".mpg",
-    ".exe",
     ".swf",
     ".woff",
     ".woff2",
@@ -182,6 +178,10 @@ set_default_static_files (void)
     ".docx",
     ".ppt ",
     ".pptx",
+    ".zip",
+    ".mp3",
+    ".mp4",
+    ".exe",
     ".iso ",
     ".gz  ",
     ".rar ",
@@ -227,6 +227,7 @@ free_cmd_args (void)
   for (i = 0; i < nargc; i++)
     free (nargv[i]);
   free (nargv);
+  free (conf.iconfigfile);
 }
 
 /* Append extra value to argv */
@@ -235,7 +236,7 @@ append_to_argv (int *argc, char ***argv, char *val)
 {
   char **_argv = xrealloc (*argv, (*argc + 2) * sizeof (*_argv));
   _argv[*argc] = val;
-  _argv[*argc + 1] = '\0';
+  _argv[*argc + 1] = (char *) '\0';
   (*argc)++;
   *argv = _argv;
 }
@@ -486,13 +487,13 @@ const char *
 verify_formats (void)
 {
   if (conf.time_format == NULL || *conf.time_format == '\0')
-    return "No time format was found on your conf file.";
+    return ERR_FORMAT_NO_TIME_FMT;
 
   if (conf.date_format == NULL || *conf.date_format == '\0')
-    return "No date format was found on your conf file.";
+    return ERR_FORMAT_NO_DATE_FMT;
 
   if (conf.log_format == NULL || *conf.log_format == '\0')
-    return "No log format was found on your conf file.";
+    return ERR_FORMAT_NO_LOG_FMT;
 
   return NULL;
 }
@@ -665,7 +666,7 @@ set_date_num_format (void)
 
   if (strpbrk (fdate, "Yy"))
     buflen += snprintf (buf + buflen, flen - buflen, "%%Y");
-  if (strpbrk (fdate, "bmB"))
+  if (strpbrk (fdate, "hbmB"))
     buflen += snprintf (buf + buflen, flen - buflen, "%%m");
   if (strpbrk (fdate, "de"))
     buflen += snprintf (buf + buflen, flen - buflen, "%%d");
