@@ -33,14 +33,14 @@
 #include <stdint.h>
 #include "commons.h"
 
-#define MAX_LINE_CONF     512
-#define MAX_EXTENSIONS    128
-#define MAX_IGNORE_IPS     64
-#define MAX_IGNORE_REF     64
-#define MAX_CUSTOM_COLORS  64
-#define MAX_IGNORE_STATUS  64
-#define MAX_OUTFORMATS      3
-#define MAX_FILENAMES     512
+#define MAX_LINE_CONF         512
+#define MAX_EXTENSIONS        128
+#define MAX_IGNORE_IPS 1024 + 128
+#define MAX_IGNORE_REF         64
+#define MAX_CUSTOM_COLORS      64
+#define MAX_IGNORE_STATUS      64
+#define MAX_OUTFORMATS          3
+#define MAX_FILENAMES         512
 #define NO_CONFIG_FILE "No config file used"
 
 typedef enum LOGTYPE
@@ -97,6 +97,7 @@ typedef struct GConf_
   const char *colors[MAX_CUSTOM_COLORS];        /* colors */
   const char *enable_panels[TOTAL_MODULES];     /* array of panels to enable */
   const char *filenames[MAX_FILENAMES];         /* log files */
+  const char *hide_referers[MAX_IGNORE_REF];    /* hide referrers from report */
   const char *ignore_ips[MAX_IGNORE_IPS];       /* array of ips to ignore */
   const char *ignore_panels[TOTAL_MODULES];     /* array of panels to ignore */
   const char *ignore_referers[MAX_IGNORE_REF];  /* referrers to ignore */
@@ -112,6 +113,8 @@ typedef struct GConf_
   char *spec_date_time_format;      /* date format w/ specificity */
   char *spec_date_time_num_format;  /* numeric date format w/ specificity */
   char *log_format;                 /* log format */
+  char *iconfigfile;                /* config file path */
+  char ***user_browsers_hash;       /* custom list of browsers */
 
   const char *debug_log;            /* debug log path */
   const char *geoip_database;       /* geoip db path */
@@ -119,8 +122,9 @@ typedef struct GConf_
   const char *html_custom_js;       /* custom JS */
   const char *html_prefs;           /* default HTML JSON preferences */
   const char *html_report_title;    /* report title */
-  const char *iconfigfile;          /* config file path */
   const char *invalid_requests_log; /* invalid lines log path */
+  const char *pidfile;              /* daemonize pid file path */
+  const char *browsers_file;        /* browser's file path */
 
   /* HTML real-time */
   const char *addr;                 /* IP address to bind to */
@@ -133,7 +137,9 @@ typedef struct GConf_
   const char *ws_url;               /* WebSocket URL */
 
   /* User flags */
+  int store_accumulated_time;       /* store accumulated processing time in tcb */
   int all_static_files;             /* parse all static files */
+  int anonymize_ip;                 /* anonymize ip addresses */
   int append_method;                /* append method to the req key */
   int append_protocol;              /* append protocol to the req key */
   int client_err_to_unique_count;   /* count 400s as visitors */
@@ -147,6 +153,7 @@ typedef struct GConf_
   int hl_header;                    /* highlight header on term */
   int ignore_crawlers;              /* ignore crawlers */
   int ignore_qstr;                  /* ignore query string */
+  int ignore_statics;               /* ignore static files */
   int json_pretty_print;            /* pretty print JSON data */
   int list_agents;                  /* show list of agents per host */
   int load_conf_dlg;                /* load curses config dialog */
@@ -157,14 +164,16 @@ typedef struct GConf_
   int no_column_names;              /* don't show col names on termnal */
   int no_csv_summary;               /* don't show overall metrics */
   int no_html_last_updated;         /* don't show HTML last updated field */
+  int no_parsing_spinner;           /* disable parsing spinner */
   int no_progress;                  /* disable progress metrics */
   int no_tab_scroll;                /* don't scroll dashboard on tab */
-  int num_tests;                    /* number of lines to test */
   int output_stdout;                /* outputting to stdout */
   int process_and_exit;             /* parse and exit without outputting */
   int real_os;                      /* show real OSs */
   int real_time_html;               /* enable real-time HTML output */
   int skip_term_resolver;           /* no terminal resolver */
+  uint32_t num_tests;               /* number of lines to test */
+  uint64_t log_size;                /* log size override */
 
   /* Internal flags */
   int bandwidth;                    /* is there bandwidth within the req line */
@@ -181,6 +190,7 @@ typedef struct GConf_
   int color_idx;                    /* colors index */
   int enable_panel_idx;             /* enable panels index */
   int filenames_idx;                /* filenames index */
+  int hide_referer_idx;             /* hide referrers index */
   int ignore_ip_idx;                /* ignored ips index */
   int ignore_panel_idx;             /* ignored panels index */
   int ignore_referer_idx;           /* ignored referrers index */
@@ -188,6 +198,7 @@ typedef struct GConf_
   int output_format_idx;            /* output format index */
   int sort_panel_idx;               /* sort panel index */
   int static_file_idx;              /* static extensions index */
+  int browsers_hash_idx;            /* browsers hash index */
 
   size_t static_file_max_len;
 
