@@ -39,7 +39,7 @@
 #include "tcbtdb.h"
 
 #ifdef HAVE_GEOLOCATION
-#include "geoip.h"
+#include "geoip1.h"
 #endif
 
 #include "error.h"
@@ -57,6 +57,13 @@ static TCADB *ht_general_stats = NULL;
 static TCADB *ht_hostnames = NULL;
 static TCADB *ht_unique_keys = NULL;
 
+/* db paths for whole app hashes */
+static char *dbpath_agent_keys = NULL;
+static char *dbpath_agent_vals = NULL;
+static char *dbpath_general_stats = NULL;
+static char *dbpath_hostnames = NULL;
+static char *dbpath_unique_keys = NULL;
+
 /* Instantiate a new store */
 static GTCStorage *
 new_tcstorage (uint32_t size)
@@ -119,8 +126,6 @@ tc_adb_create (char *path)
     FATAL ("Unable to open an abstract database: %s", params);
   }
 
-  free (path);
-
   return adb;
 }
 
@@ -136,20 +141,45 @@ get_dbname (const char *dbname, int module)
 #endif
 
 #ifdef TCB_BTREE
-  char *db = NULL;
   path = tc_db_set_path (dbname, module);
-
-  if (module >= 0) {
-    db = xmalloc (snprintf (NULL, 0, "%s", path) + 1);
-    sprintf (db, "%s", path);
-    free (path);
-    return db;
-  }
 #endif
 
   return path;
 }
 
+/* Create hashes used across the whole app (not per module) */
+static void
+create_prog_tables (void)
+{
+  ht_agent_keys = tc_adb_create (dbpath_agent_keys);
+  ht_agent_vals = tc_adb_create (dbpath_agent_vals);
+  ht_general_stats = tc_adb_create (dbpath_general_stats);
+  ht_hostnames = tc_adb_create (dbpath_hostnames);
+  ht_unique_keys = tc_adb_create (dbpath_unique_keys);
+}
+
+/* Set db paths for hashes used across the whole app (not per module) */
+static void
+set_prog_tables_dbpaths (void)
+{
+  dbpath_agent_keys = get_dbname (DB_AGENT_KEYS, -1);
+  dbpath_agent_vals = get_dbname (DB_AGENT_VALS, -1);
+  dbpath_general_stats = get_dbname (DB_GEN_STATS, -1);
+  dbpath_hostnames = get_dbname (DB_HOSTNAMES, -1);
+  dbpath_unique_keys = get_dbname (DB_UNIQUE_KEYS, -1);
+}
+
+/* Free hashes used across the whole app (not per module) */
+static void
+free_prog_tables (void)
+{
+  tc_db_close (ht_agent_keys, dbpath_agent_keys);
+  tc_db_close (ht_agent_vals, dbpath_agent_vals);
+  tc_db_close (ht_general_stats, dbpath_general_stats);
+  tc_db_close (ht_hostnames, dbpath_hostnames);
+  tc_db_close (ht_unique_keys, dbpath_unique_keys);
+}
+
 /* Initialize map & metric hashes */
 static void
 init_tables (GModule module)
@@ -159,20 +189,20 @@ init_tables (GModule module)
 
   /* *INDENT-OFF* */
   GTCStorageMetric metrics[] = {
-    {MTRC_KEYMAP    , DB_KEYMAP    , NULL} ,
-    {MTRC_ROOTMAP   , DB_ROOTMAP   , NULL} ,
-    {MTRC_DATAMAP   , DB_DATAMAP   , NULL} ,
-    {MTRC_UNIQMAP   , DB_UNIQMAP   , NULL} ,
-    {MTRC_ROOT      , DB_ROOT      , NULL} ,
-    {MTRC_HITS      , DB_HITS      , NULL} ,
-    {MTRC_VISITORS  , DB_VISITORS  , NULL} ,
-    {MTRC_BW        , DB_BW        , NULL} ,
-    {MTRC_CUMTS     , DB_CUMTS     , NULL} ,
-    {MTRC_MAXTS     , DB_MAXTS     , NULL} ,
-    {MTRC_METHODS   , DB_METHODS   , NULL} ,
-    {MTRC_PROTOCOLS , DB_PROTOCOLS , NULL} ,
-    {MTRC_AGENTS    , DB_AGENTS    , NULL} ,
-    {MTRC_METADATA  , DB_METADATA  , NULL} ,
+    {MTRC_KEYMAP    , DB_KEYMAP    , NULL, NULL} ,
+    {MTRC_ROOTMAP   , DB_ROOTMAP   , NULL, NULL} ,
+    {MTRC_DATAMAP   , DB_DATAMAP   , NULL, NULL} ,
+    {MTRC_UNIQMAP   , DB_UNIQMAP   , NULL, NULL} ,
+    {MTRC_ROOT      , DB_ROOT      , NULL, NULL} ,
+    {MTRC_HITS      , DB_HITS      , NULL, NULL} ,
+    {MTRC_VISITORS  , DB_VISITORS  , NULL, NULL} ,
+    {MTRC_BW        , DB_BW        , NULL, NULL} ,
+    {MTRC_CUMTS     , DB_CUMTS     , NULL, NULL} ,
+    {MTRC_MAXTS     , DB_MAXTS     , NULL, NULL} ,
+    {MTRC_METHODS   , DB_METHODS   , NULL, NULL} ,
+    {MTRC_PROTOCOLS , DB_PROTOCOLS , NULL, NULL} ,
+    {MTRC_AGENTS    , DB_AGENTS    , NULL, NULL} ,
+    {MTRC_METADATA  , DB_METADATA  , NULL, NULL} ,
   };
   /* *INDENT-ON* */
 
@@ -180,14 +210,18 @@ init_tables (GModule module)
   for (i = 0; i < n; i++) {
     mtrc = metrics[i];
 #ifdef TCB_MEMHASH
-    mtrc.store = tc_adb_create (get_dbname (mtrc.dbname, module));
+    mtrc.dbpath = get_dbname (mtrc.dbname, module);
+    mtrc.store = tc_adb_create (mtrc.dbpath);
 #endif
 #ifdef TCB_BTREE
     /* allow for duplicate keys */
-    if (mtrc.metric == MTRC_AGENTS)
-      mtrc.store = tc_bdb_create (DB_AGENTS, module);
-    else
-      mtrc.store = tc_adb_create (get_dbname (mtrc.dbname, module));
+    if (mtrc.metric == MTRC_AGENTS) {
+      mtrc.dbpath = tc_db_set_path (DB_AGENTS, module);
+      mtrc.store = tc_bdb_create (mtrc.dbpath);
+    } else {
+      mtrc.dbpath = get_dbname (mtrc.dbname, module);
+      mtrc.store = tc_adb_create (mtrc.dbpath);
+    }
 #endif
     tc_storage[module].metrics[i] = mtrc;
   }
@@ -200,12 +234,8 @@ init_storage (void)
   GModule module;
   size_t idx = 0;
 
-  /* Hashes used across the whole app (not per module) */
-  ht_agent_keys = tc_adb_create (get_dbname (DB_AGENT_KEYS, -1));
-  ht_agent_vals = tc_adb_create (get_dbname (DB_AGENT_VALS, -1));
-  ht_general_stats = tc_adb_create (get_dbname (DB_GEN_STATS, -1));
-  ht_hostnames = tc_adb_create (get_dbname (DB_HOSTNAMES, -1));
-  ht_unique_keys = tc_adb_create (get_dbname (DB_UNIQUE_KEYS, -1));
+  set_prog_tables_dbpaths ();
+  create_prog_tables ();
 
   tc_storage = new_tcstorage (TOTAL_MODULES);
 
@@ -227,13 +257,13 @@ free_metrics (GModule module)
   for (i = 0; i < GSMTRC_TOTAL; i++) {
     mtrc = tc_storage[module].metrics[i];
 #ifdef TCB_MEMHASH
-    tc_db_close (mtrc.store, get_dbname (mtrc.dbname, module));
+    tc_db_close (mtrc.store, mtrc.dbpath);
 #endif
 #ifdef TCB_BTREE
     if (mtrc.metric == MTRC_AGENTS)
-      tc_bdb_close (mtrc.store, get_dbname (mtrc.dbname, module));
+      tc_bdb_close (mtrc.store, mtrc.dbpath);
     else
-      tc_db_close (mtrc.store, get_dbname (mtrc.dbname, module));
+      tc_db_close (mtrc.store, mtrc.dbpath);
 #endif
   }
 }
@@ -247,16 +277,16 @@ free_storage (void)
   if (!tc_storage)
     return;
 
-  tc_db_close (ht_agent_keys, get_dbname (DB_AGENT_KEYS, -1));
-  tc_db_close (ht_agent_vals, get_dbname (DB_AGENT_VALS, -1));
-  tc_db_close (ht_general_stats, get_dbname (DB_GEN_STATS, -1));
-  tc_db_close (ht_hostnames, get_dbname (DB_HOSTNAMES, -1));
-  tc_db_close (ht_unique_keys, get_dbname (DB_UNIQUE_KEYS, -1));
-
+  free_prog_tables ();
   FOREACH_MODULE (idx, module_list) {
     free_metrics (module_list[idx]);
   }
   free (tc_storage);
+
+#ifdef TCB_BTREE
+  tc_db_rmdir ();
+#endif
+
 }
 
 static uint32_t
@@ -432,13 +462,13 @@ inc_iu64 (void *hash, int key, uint64_t inc)
 static int
 inc_si32 (void *hash, const char *key, int inc)
 {
-  int value = inc;
+  int value = inc, sp;
   void *ptr;
 
   if (!hash)
     return -1;
 
-  if ((ptr = tcadbget2 (hash, key)) != NULL) {
+  if ((ptr = tcadbget (hash, key, strlen (key), &sp)) != NULL) {
     value = (*(int *) ptr) + inc;
     free (ptr);
   }
@@ -453,17 +483,18 @@ inc_si32 (void *hash, const char *key, int inc)
 /* Insert a string key and auto increment by uint64_t value.
  *
  * On error, -1 is returned.
- * On success the value of the key inserted is returned */
+ * On success the value of the key is inserted and 0 is returned */
 static int
 inc_su64 (void *hash, const char *key, uint64_t inc)
 {
+  int sp;
   uint64_t value = inc;
   void *ptr;
 
   if (!hash)
     return -1;
 
-  if ((ptr = tcadbget2 (hash, key)) != NULL) {
+  if ((ptr = tcadbget (hash, key, strlen (key), &sp)) != NULL) {
     value = (*(uint64_t *) ptr) + inc;
     free (ptr);
   }
@@ -543,19 +574,19 @@ out:
 
 /* Get the int value of a given string key.
  *
- * On error, -1 is returned.
+ * On error, or if key is not found, -1 is returned.
  * On success the int value for the given key is returned */
 static int
 get_si32 (void *hash, const char *key)
 {
-  int ret = 0;
+  int ret = 0, sp;
   void *ptr;
 
   if (!hash)
     return -1;
 
   /* key found, return current value */
-  if ((ptr = tcadbget2 (hash, key)) != NULL) {
+  if ((ptr = tcadbget (hash, key, strlen (key), &sp)) != NULL) {
     ret = (*(int *) ptr);
     free (ptr);
     return ret;
@@ -566,18 +597,18 @@ get_si32 (void *hash, const char *key)
 
 /* Get the unsigned int value of a given string key.
  *
- * On error, 0 is returned.
- * On success the int value for the given key is returned */
+ * On error, or if key is not found, 0 is returned.
+ * On success the uint32_t value for the given key is returned */
 static uint32_t
 get_sui32 (void *hash, const char *key)
 {
-  int ret = 0;
+  int ret = 0, sp;
   void *ptr;
   if (!hash)
     return 0;
 
   /* key found, return current value */
-  if ((ptr = tcadbget2 (hash, key)) != NULL) {
+  if ((ptr = tcadbget (hash, key, strlen (key), &sp)) != NULL) {
     ret = (*(uint32_t *) ptr);
     free (ptr);
     return ret;
@@ -588,18 +619,19 @@ get_sui32 (void *hash, const char *key)
 
 /* Get the uint64_t value of a given string key.
  *
- * On error, 0 is returned.
- * On success the int value for the given key is returned */
+ * On error, or if key is not found, 0 is returned.
+ * On success the uint64_t value for the given key is returned */
 static uint64_t
 get_su64 (void *hash, const char *key)
 {
+  int sp;
   uint64_t ret = 0;
   void *ptr;
   if (!hash)
     return 0;
 
   /* key found, return current value */
-  if ((ptr = tcadbget2 (hash, key)) != NULL) {
+  if ((ptr = tcadbget (hash, key, strlen (key), &sp)) != NULL) {
     ret = (*(uint64_t *) ptr);
     free (ptr);
     return ret;
@@ -679,7 +711,7 @@ get_iu64_min_max (void *hash, uint64_t * min, uint64_t * max)
 
 /* Get the string value of a given int key.
  *
- * On error, NULL is returned.
+ * On error, or if key is not found, NULL is returned.
  * On success the string value for the given key is returned */
 static char *
 get_is32 (void *hash, int key)
@@ -698,7 +730,7 @@ get_is32 (void *hash, int key)
 
 /* Get the string value of a given string key.
  *
- * On error, NULL is returned.
+ * On error, or if key is not found, NULL is returned.
  * On success the string value for the given key is returned */
 static char *
 get_ss32 (void *hash, const char *key)
@@ -1192,6 +1224,21 @@ ht_replace_genstats (const char *key, int value)
   return ins_si32 (hash, key, value);
 }
 
+/* Increases the general stats counter accumulated_time.
+ *
+ * On error,  0 is returned
+ * On success 1 is returned */
+int
+ht_insert_genstats_accumulated_time (time_t elapsed)
+{
+  void *hash = ht_general_stats;
+
+  if (!hash)
+    return 0;
+
+  return inc_si32 (hash, "accumulated_time", (int) elapsed) != -1;
+}
+
 /* Increases a general stats counter uint64_t from a string key.
  *
  * On error, -1 is returned.
