@@ -35,15 +35,19 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <tcutil.h>
+#include <fcntl.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 #include "tcbtdb.h"
 #include "tcabdb.h"
 
 #ifdef HAVE_GEOLOCATION
-#include "geoip.h"
+#include "geoip1.h"
 #endif
 
 #include "error.h"
+#include "util.h"
 #include "xmalloc.h"
 
 #ifdef TCB_BTREE
@@ -54,31 +58,57 @@
 char *
 tc_db_set_path (const char *dbname, int module)
 {
+  struct stat info;
+  static char default_path[sizeof (TC_DBPATH "01234567890/")];
+  static char *db_path;
   char *path;
+  char fname[RAND_FN];
   int cx;
 
-  struct stat info;
-
-  if (conf.db_path != NULL) {
-    /* sanity check: Is db_path accessible and a directory? */
-    if (stat (conf.db_path, &info) != 0) {
-      FATAL ("Unable to access database path: %s", strerror (errno));
-    } else if (!(info.st_mode & S_IFDIR)) {
-      FATAL ("Database path is not a directory.");
-    }
-
-    cx = snprintf (NULL, 0, "%s%dm%s", conf.db_path, module, dbname) + 1;
-    path = xmalloc (cx);
-    sprintf (path, "%s%dm%s", conf.db_path, module, dbname);
-  } else {
-    cx = snprintf (NULL, 0, "%s%dm%s", TC_DBPATH, module, dbname) + 1;
-    path = xmalloc (cx);
-    sprintf (path, "%s%dm%s", TC_DBPATH, module, dbname);
+  /* db_path is either specified explicitly, or gets the default (pid appended) */
+  if (conf.db_path != NULL)
+    db_path = (char *) conf.db_path;
+  else {
+    sprintf (default_path, "%s%d", TC_DBPATH, getpid ());
+    db_path = default_path;
+    mkdir (db_path, TC_DBPMODE);
   }
 
+  /* sanity check: Is db_path accessible and a directory? */
+  if (stat (db_path, &info) != 0)
+    FATAL ("Unable to access database path: %s", strerror (errno));
+  if (!(info.st_mode & S_IFDIR))
+    FATAL ("Database path is not a directory.");
+
+  /* for tc_db_rmdir(), return the pure folder path (or NULL to keep) */
+  if (dbname == NULL)
+    return conf.db_path == NULL ? db_path : NULL;
+
+  memset (fname, 0, sizeof (fname));
+  /* tcadbopen requires the db name suffix to be ".tcb" and thus we
+   * don't use mkstemp(3) */
+  genstr (fname, RAND_FN - 1);
+
+  cx = snprintf (NULL, 0, "%s/%dm%s%s", db_path, module, fname, dbname) + 1;
+  path = xmalloc (cx);
+  sprintf (path, "%s/%dm%s%s", db_path, module, fname, dbname);
+
   return path;
 }
 
+/* delete db folder if we used the customized (pid appended) default */
+void
+tc_db_rmdir ()
+{
+  const char *db_path = NULL;
+
+  db_path = tc_db_set_path (NULL, 0);
+  if (db_path != NULL)
+    if (rmdir (db_path))
+      LOG_DEBUG (("Unable to remove custom db folder: %s\n", db_path));
+}
+
+
 /* Set the given database parameter into the parameters buffer.
  *
  * On error, a negative number is returned.
@@ -160,14 +190,12 @@ tc_db_get_params (char *params, const char *path)
  * On error, the program will exit.
  * On success, the opened on-disk database is returned. */
 TCBDB *
-tc_bdb_create (const char *dbname, int module)
+tc_bdb_create (char *dbpath)
 {
   TCBDB *bdb;
-  char *path = NULL;
   int ecode;
   uint32_t lcnum, ncnum, lmemb, nmemb, bnum, flags;
 
-  path = tc_db_set_path (dbname, module);
   bdb = tcbdbnew ();
 
   lcnum = conf.cache_lcnum > 0 ? conf.cache_lcnum : TC_LCNUM;
@@ -175,13 +203,13 @@ tc_bdb_create (const char *dbname, int module)
 
   /* set the caching parameters of a B+ tree database object */
   if (!tcbdbsetcache (bdb, lcnum, ncnum)) {
-    free (path);
+    free (dbpath);
     FATAL ("Unable to set TCB cache");
   }
 
   /* set the size of the extra mapped memory */
   if (conf.xmmap > 0 && !tcbdbsetxmsiz (bdb, conf.xmmap)) {
-    free (path);
+    free (dbpath);
     FATAL ("Unable to set TCB xmmap.");
   }
 
@@ -206,13 +234,12 @@ tc_bdb_create (const char *dbname, int module)
     flags |= BDBOTRUNC;
 
   /* attempt to open the database */
-  if (!tcbdbopen (bdb, path, flags)) {
-    free (path);
+  if (!tcbdbopen (bdb, dbpath, flags)) {
+    free (dbpath);
     ecode = tcbdbecode (bdb);
 
     FATAL ("%s", tcbdberrmsg (ecode));
   }
-  free (path);
 
   return bdb;
 }
