@@ -46,10 +46,13 @@
 #define TC_LMEMB 128
 #define TC_NMEMB 256
 #define TC_BNUM  32749
-#define TC_DBPATH "/tmp/"
+#define TC_DBPATH "/tmp/goaccess"
+#define TC_DBPMODE 0755
 #define TC_ZLIB 1
 #define TC_BZ2  2
 
+#define RAND_FN 7 + 1
+
 /* B+ Tree - on-disk databases */
 #define DB_AGENT_KEYS  "db_agent_keys.tcb"
 #define DB_AGENT_VALS  "db_agent_vals.tcb"
@@ -73,9 +76,10 @@
 #define DB_METADATA  "db_metadata.tcb"
 
 /* *INDENT-OFF* */
-TCBDB *tc_bdb_create (const char *dbname, int module);
+TCBDB *tc_bdb_create (char *dbpath);
 
 char *tc_db_set_path (const char *dbname, int module);
+void tc_db_rmdir(void);
 int tc_bdb_close (void *db, char *dbname);
 void tc_db_get_params (char *params, const char *path);
 
