@@ -175,6 +175,9 @@ generate_time (void)
 void
 end_spinner (void)
 {
+  if (conf.no_parsing_spinner)
+    return;
+
   pthread_mutex_lock (&parsing_spinner->mutex);
   parsing_spinner->state = SPN_END;
   pthread_mutex_unlock (&parsing_spinner->mutex);
@@ -262,26 +265,26 @@ const char *
 module_to_label (GModule module)
 {
   static const char *modules[] = {
-    VISIT_LABEL,
-    REQUE_LABEL,
-    STATI_LABEL,
-    FOUND_LABEL,
+    VISITORS_LABEL,
+    REQUESTS_LABEL,
+    REQUESTS_STATIC_LABEL,
+    NOT_FOUND_LABEL,
     HOSTS_LABEL,
-    OPERA_LABEL,
-    BROWS_LABEL,
-    VTIME_LABEL,
-    VHOST_LABEL,
-    REFER_LABEL,
-    SITES_LABEL,
-    KEYPH_LABEL,
-    CODES_LABEL,
-    RUSER_LABEL,
+    OS_LABEL,
+    BROWSERS_LABEL,
+    VISIT_TIMES_LABEL,
+    VIRTUAL_HOSTS_LABEL,
+    REFERRERS_LABEL,
+    REFERRING_SITES_LABEL,
+    KEYPHRASES_LABEL,
+    STATUS_CODES_LABEL,
+    REMOTE_USER_LABEL,
 #ifdef HAVE_GEOLOCATION
-    GEOLO_LABEL,
+    GEO_LOCATION_LABEL,
 #endif
   };
 
-  return modules[module];
+  return _(modules[module]);
 }
 
 /* Get the module/panel label id for the given module enum value.
@@ -291,26 +294,26 @@ const char *
 module_to_id (GModule module)
 {
   static const char *modules[] = {
-    VISIT_ID,
-    REQUE_ID,
-    STATI_ID,
-    FOUND_ID,
+    VISITORS_ID,
+    REQUESTS_ID,
+    REQUESTS_STATIC_ID,
+    NOT_FOUND_ID,
     HOSTS_ID,
-    OPERA_ID,
-    BROWS_ID,
-    VTIME_ID,
-    VHOST_ID,
-    REFER_ID,
-    SITES_ID,
-    KEYPH_ID,
-    CODES_ID,
-    RUSER_ID,
+    OS_ID,
+    BROWSERS_ID,
+    VISIT_TIMES_ID,
+    VIRTUAL_HOSTS_ID,
+    REFERRERS_ID,
+    REFERRING_SITES_ID,
+    KEYPHRASES_ID,
+    STATUS_CODES_ID,
+    REMOTE_USER_ID,
 #ifdef HAVE_GEOLOCATION
-    GEOLO_ID,
+    GEO_LOCATION_ID,
 #endif
   };
 
-  return modules[module];
+  return _(modules[module]);
 }
 
 /* Get the module/panel label header for the given module enum value.
@@ -320,29 +323,29 @@ const char *
 module_to_head (GModule module)
 {
   static const char *modules[] = {
-    VISIT_HEAD,
-    REQUE_HEAD,
-    STATI_HEAD,
-    FOUND_HEAD,
+    VISITORS_HEAD,
+    REQUESTS_HEAD,
+    REQUESTS_STATIC_HEAD,
+    NOT_FOUND_HEAD,
     HOSTS_HEAD,
-    OPERA_HEAD,
-    BROWS_HEAD,
-    VTIME_HEAD,
-    VHOST_HEAD,
-    REFER_HEAD,
-    SITES_HEAD,
-    KEYPH_HEAD,
-    CODES_HEAD,
-    RUSER_HEAD,
+    OS_HEAD,
+    BROWSERS_HEAD,
+    VISIT_TIMES_HEAD,
+    VIRTUAL_HOSTS_HEAD,
+    REFERRERS_HEAD,
+    REFERRING_SITES_HEAD,
+    KEYPHRASES_HEAD,
+    STATUS_CODES_HEAD,
+    REMOTE_USER_HEAD,
 #ifdef HAVE_GEOLOCATION
-    GEOLO_HEAD,
+    GEO_LOCATION_HEAD,
 #endif
   };
 
   if (!conf.ignore_crawlers)
-    modules[VISITORS] = VISIT_HEAD INCLUDE_BOTS;
+    modules[VISITORS] = VISITORS_HEAD_BOTS;
 
-  return modules[module];
+  return _(modules[module]);
 }
 
 /* Get the module/panel label description for the given module enum
@@ -353,26 +356,26 @@ const char *
 module_to_desc (GModule module)
 {
   static const char *modules[] = {
-    VISIT_DESC,
-    REQUE_DESC,
-    STATI_DESC,
-    FOUND_DESC,
+    VISITORS_DESC,
+    REQUESTS_DESC,
+    REQUESTS_STATIC_DESC,
+    NOT_FOUND_DESC,
     HOSTS_DESC,
-    OPERA_DESC,
-    BROWS_DESC,
-    VTIME_DESC,
-    VHOST_DESC,
-    REFER_DESC,
-    SITES_DESC,
-    KEYPH_DESC,
-    CODES_DESC,
-    RUSER_DESC,
+    OS_DESC,
+    BROWSERS_DESC,
+    VISIT_TIMES_DESC,
+    VIRTUAL_HOSTS_DESC,
+    REFERRERS_DESC,
+    REFERRING_SITES_DESC,
+    KEYPHRASES_DESC,
+    STATUS_CODES_DESC,
+    REMOTE_USER_DESC,
 #ifdef HAVE_GEOLOCATION
-    GEOLO_DESC,
+    GEO_LOCATION_DESC,
 #endif
   };
 
-  return modules[module];
+  return _(modules[module]);
 }
 
 /* Rerender the header window to reflect active module. */
@@ -383,8 +386,8 @@ update_active_module (WINDOW * header_win, GModule current)
   const char *module = module_to_label (current);
   int col = getmaxx (stdscr);
 
-  char *lbl = xmalloc (snprintf (NULL, 0, "[Active Panel: %s]", module) + 1);
-  sprintf (lbl, "[Active Panel: %s]", module);
+  char *lbl = xmalloc (snprintf (NULL, 0, T_ACTIVE_PANEL, module) + 1);
+  sprintf (lbl, T_ACTIVE_PANEL, module);
 
   wmove (header_win, 0, 30);
 
@@ -506,8 +509,15 @@ get_str_visitors (void)
 static char *
 get_str_proctime (void)
 {
+  char *s = NULL;
   uint64_t secs = (long long) end_proc - start_proc;
-  char *s = xmalloc (snprintf (NULL, 0, "%" PRIu64 "s", secs) + 1);
+
+#ifdef TCB_BTREE
+  if (conf.store_accumulated_time)
+    secs = (uint64_t) ht_get_genstats ("accumulated_time");
+#endif
+
+  s = xmalloc (snprintf (NULL, 0, "%" PRIu64 "s", secs) + 1);
   sprintf (s, "%" PRIu64 "s", secs);
 
   return s;
@@ -591,7 +601,7 @@ get_start_end_parsing_dates (GHolder * h, char **start, char **end,
 char *
 get_overall_header (GHolder * h)
 {
-  const char *head = T_DASH " - " T_HEAD;
+  const char *head = T_DASH_HEAD;
   char *hd = NULL, *start = NULL, *end = NULL;
 
   if (h->idx == 0 || get_start_end_parsing_dates (h, &start, &end, "%d/%b/%Y"))
@@ -682,19 +692,19 @@ display_general (WINDOW * win, GLog * glog, GHolder * h)
 
   /* *INDENT-OFF* */
   Field fields[] = {
-    {T_REQUESTS   , get_str_processed_reqs (glog)  , colorlbl , colorval , 0} ,
-    {T_UNIQUE_VIS , get_str_visitors ()            , colorlbl , colorval , 0} ,
-    {T_UNIQUE_FIL , get_str_reqs ()                , colorlbl , colorval , 0} ,
-    {T_REFERRER   , get_str_ref_reqs ()            , colorlbl , colorval , 0} ,
-    {T_VALID      , get_str_valid_reqs (glog)      , colorlbl , colorval , 0} ,
-    {T_GEN_TIME   , get_str_proctime ()            , colorlbl , colorval , 0} ,
-    {T_STATIC_FIL , get_str_static_reqs ()         , colorlbl , colorval , 0} ,
-    {T_LOG        , get_str_filesize ()            , colorlbl , colorval , 0} ,
-    {T_FAILED     , get_str_failed_reqs (glog)     , colorlbl , colorval , 0} ,
-    {T_EXCLUDE_IP , get_str_excluded_ips (glog)    , colorlbl , colorval , 0} ,
-    {T_UNIQUE404  , get_str_notfound_reqs ()       , colorlbl , colorval , 0} ,
-    {T_BW         , get_str_bandwidth (glog)       , colorlbl , colorval , 0} ,
-    {T_LOG_PATH   , get_str_logfile ()             , colorlbl , colorpth , 1}
+    {T_REQUESTS        , get_str_processed_reqs (glog), colorlbl, colorval, 0},
+    {T_UNIQUE_VISITORS , get_str_visitors ()          , colorlbl, colorval, 0},
+    {T_UNIQUE_FILES    , get_str_reqs ()              , colorlbl, colorval, 0},
+    {T_REFERRER        , get_str_ref_reqs ()          , colorlbl, colorval, 0},
+    {T_VALID           , get_str_valid_reqs (glog)    , colorlbl, colorval, 0},
+    {T_GEN_TIME        , get_str_proctime ()          , colorlbl, colorval, 0},
+    {T_STATIC_FILES    , get_str_static_reqs ()       , colorlbl, colorval, 0},
+    {T_LOG             , get_str_filesize ()          , colorlbl, colorval, 0},
+    {T_FAILED          , get_str_failed_reqs (glog)   , colorlbl, colorval, 0},
+    {T_EXCLUDE_IP      , get_str_excluded_ips (glog)  , colorlbl, colorval, 0},
+    {T_UNIQUE404       , get_str_notfound_reqs ()     , colorlbl, colorval, 0},
+    {T_BW              , get_str_bandwidth (glog)     , colorlbl, colorval, 0},
+    {T_LOG_PATH        , get_str_logfile ()           , colorlbl, colorpth, 1}
   };
   /* *INDENT-ON* */
 
@@ -987,9 +997,9 @@ load_agent_list (WINDOW * main_win, char *addr)
     goto out;
 
   post_gmenu (menu);
-  snprintf (buf, sizeof buf, "User Agents for %s", addr);
+  snprintf (buf, sizeof buf, AGENTSDLG_HEAD, addr);
   draw_header (win, buf, " %s", 1, 1, list_w - 2, color_panel_header);
-  mvwprintw (win, 2, 2, "[UP/DOWN] to scroll - [q] to close window");
+  mvwprintw (win, 2, 2, AGENTSDLG_DESC);
   wrefresh (win);
 
   while (quit) {
@@ -1077,6 +1087,9 @@ ui_spinner (void *ptr_data)
 void
 ui_spinner_create (GSpinner * spinner)
 {
+  if (conf.no_parsing_spinner)
+    return;
+
   pthread_create (&(spinner->thread), NULL, (void *) &ui_spinner, spinner);
   pthread_detach (spinner->thread);
 }
@@ -1145,7 +1158,7 @@ get_input_date_format (void)
 
 /* Escape a time format string.
  *
- * If no conf.date_format is given, NULL is returned.
+ * If no conf.time_format is given, NULL is returned.
  * On success, the newly escaped allocated string is returned. */
 static char *
 get_input_time_format (void)
@@ -1159,7 +1172,7 @@ get_input_time_format (void)
 
 /* Escape a log format string.
  *
- * If no conf.date_format is given, NULL is returned.
+ * If no conf.log_format is given, NULL is returned.
  * On success, the newly escaped allocated string is returned. */
 static char *
 get_input_log_format (void)
@@ -1176,13 +1189,11 @@ draw_formats (WINDOW * win, int w2)
 {
   char *date_format = NULL, *log_format = NULL, *time_format = NULL;
 
-  draw_header (win, "Log Format Configuration", " %s", 1, 1, w2,
-               color_panel_header);
-  mvwprintw (win, 2, 2, "[SPACE] to toggle - [ENTER] to proceed - [q]uit");
+  draw_header (win, CONFDLG_HEAD, " %s", 1, 1, w2, color_panel_header);
+  mvwprintw (win, 2, 2, CONFDLG_KEY_HINTS);
 
   /* set log format from config file if available */
-  draw_header (win, "Log Format - [c] to add/edit format", " %s", 11, 1, w2,
-               color_panel_header);
+  draw_header (win, CONFDLG_LOG_FORMAT, " %s", 11, 1, w2, color_panel_header);
   if ((log_format = get_input_log_format ())) {
     mvwprintw (win, 12, 2, "%.*s", CONF_MENU_W, log_format);
 
@@ -1190,8 +1201,7 @@ draw_formats (WINDOW * win, int w2)
   }
 
   /* set log format from config file if available */
-  draw_header (win, "Date Format - [d] to add/edit format", " %s", 14, 1, w2,
-               color_panel_header);
+  draw_header (win, CONFDLG_DATE_FORMAT, " %s", 14, 1, w2, color_panel_header);
   if ((date_format = get_input_date_format ())) {
     mvwprintw (win, 15, 2, "%.*s", CONF_MENU_W, date_format);
 
@@ -1199,8 +1209,7 @@ draw_formats (WINDOW * win, int w2)
   }
 
   /* set log format from config file if available */
-  draw_header (win, "Time Format - [t] to add/edit format", " %s", 17, 1, w2,
-               color_panel_header);
+  draw_header (win, CONFDLG_TIME_FORMAT, " %s", 17, 1, w2, color_panel_header);
   if ((time_format = get_input_time_format ())) {
     mvwprintw (win, 18, 2, "%.*s", CONF_MENU_W, time_format);
 
@@ -1213,11 +1222,11 @@ set_formats (char *date_format, char *log_format, char *time_format)
 {
   /* display status bar error messages */
   if (!time_format && !conf.time_format)
-    return "Select a time format.";
+    return ERR_FORMAT_NO_TIME_FMT_DLG;
   if (!date_format && !conf.date_format)
-    return "Select a date format.";
+    return ERR_FORMAT_NO_DATE_FMT_DLG;
   if (!log_format && !conf.log_format)
-    return "Select a log format.";
+    return ERR_FORMAT_NO_LOG_FMT_DLG;
 
   if (time_format) {
     free (conf.time_format);
@@ -1269,8 +1278,8 @@ load_confdlg_error (WINDOW * parent_win, char **errors, int nerrors)
   free (errors);
   post_gmenu (menu);
 
-  draw_header (win, ERR_HEADER, " %s", 1, 1, w - 2, color_error);
-  mvwprintw (win, 2, 2, "[UP/DOWN] to scroll - [q] to quit");
+  draw_header (win, ERR_FORMAT_HEADER, " %s", 1, 1, w - 2, color_error);
+  mvwprintw (win, 2, 2, CONFDLG_DESC);
 
   wrefresh (win);
   while (quit) {
@@ -1608,9 +1617,8 @@ load_schemes_win (WINDOW * main_win)
   }
   post_gmenu (menu);
 
-  draw_header (win, "Scheme Configuration", " %s", 1, 1, w2,
-               color_panel_header);
-  mvwprintw (win, 2, 2, "[ENTER] to use scheme - [q]uit");
+  draw_header (win, SCHEMEDLG_HEAD, " %s", 1, 1, w2, color_panel_header);
+  mvwprintw (win, 2, 2, SCHEMEDLG_DESC);
 
   wrefresh (win);
   while (quit) {
@@ -1704,55 +1712,55 @@ load_sort_win (WINDOW * main_win, GModule module, GSort * sort)
   for (i = 0; i < n; ++i) {
     GSortField field = sort_choices[module][opts[i]];
     if (SORT_BY_HITS == field) {
-      menu->items[i].name = alloc_string ("Hits");
+      menu->items[i].name = alloc_string (MTRC_HITS_LBL);
       if (sort->field == SORT_BY_HITS) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_VISITORS == field) {
-      menu->items[i].name = alloc_string ("Visitors");
+      menu->items[i].name = alloc_string (MTRC_VISITORS_LBL);
       if (sort->field == SORT_BY_VISITORS) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_DATA == field) {
-      menu->items[i].name = alloc_string ("Data");
+      menu->items[i].name = alloc_string (MTRC_DATA_LBL);
       if (sort->field == SORT_BY_DATA) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_BW == field) {
-      menu->items[i].name = alloc_string ("Bandwidth");
+      menu->items[i].name = alloc_string (MTRC_BW_LBL);
       if (sort->field == SORT_BY_BW) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_AVGTS == field) {
-      menu->items[i].name = alloc_string ("Avg. Time Served");
+      menu->items[i].name = alloc_string (MTRC_AVGTS_LBL);
       if (sort->field == SORT_BY_AVGTS) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_CUMTS == field) {
-      menu->items[i].name = alloc_string ("Cum. Time Served");
+      menu->items[i].name = alloc_string (MTRC_CUMTS_LBL);
       if (sort->field == SORT_BY_CUMTS) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_MAXTS == field) {
-      menu->items[i].name = alloc_string ("Max. Time Served");
+      menu->items[i].name = alloc_string (MTRC_MAXTS_LBL);
       if (sort->field == SORT_BY_MAXTS) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_PROT == field) {
-      menu->items[i].name = alloc_string ("Protocol");
+      menu->items[i].name = alloc_string (MTRC_PROTOCOLS_LBL);
       if (sort->field == SORT_BY_PROT) {
         menu->items[i].checked = 1;
         menu->idx = i;
       }
     } else if (SORT_BY_MTHD == field) {
-      menu->items[i].name = alloc_string ("Method");
+      menu->items[i].name = alloc_string (MTRC_METHODS_LBL);
       if (sort->field == SORT_BY_MTHD) {
         menu->items[i].checked = 1;
         menu->idx = i;
@@ -1761,9 +1769,8 @@ load_sort_win (WINDOW * main_win, GModule module, GSort * sort)
   }
   post_gmenu (menu);
 
-  draw_header (win, "Sort active module by", " %s", 1, 1, w2,
-               color_panel_header);
-  mvwprintw (win, 2, 2, "[ENTER] select - [TAB] sort - [q]uit");
+  draw_header (win, SORTDLG_HEAD, " %s", 1, 1, w2, color_panel_header);
+  mvwprintw (win, 2, 2, SORTDLG_DESC);
 
   if (sort->sort == SORT_ASC)
     mvwprintw (win, SORT_WIN_H - 2, 1, " %s", SORT_ASC_SEL);
@@ -1805,13 +1812,13 @@ load_sort_win (WINDOW * main_win, GModule module, GSort * sort)
           sort->field = SORT_BY_VISITORS;
         else if (strcmp ("Data", menu->items[i].name) == 0)
           sort->field = SORT_BY_DATA;
-        else if (strcmp ("Bandwidth", menu->items[i].name) == 0)
+        else if (strcmp ("Tx. Amount", menu->items[i].name) == 0)
           sort->field = SORT_BY_BW;
-        else if (strcmp ("Avg. Time Served", menu->items[i].name) == 0)
+        else if (strcmp ("Avg. T.S.", menu->items[i].name) == 0)
           sort->field = SORT_BY_AVGTS;
-        else if (strcmp ("Cum. Time Served", menu->items[i].name) == 0)
+        else if (strcmp ("Cum. T.S.", menu->items[i].name) == 0)
           sort->field = SORT_BY_CUMTS;
-        else if (strcmp ("Max. Time Served", menu->items[i].name) == 0)
+        else if (strcmp ("Max. T.S.", menu->items[i].name) == 0)
           sort->field = SORT_BY_MAXTS;
         else if (strcmp ("Protocol", menu->items[i].name) == 0)
           sort->field = SORT_BY_PROT;
@@ -1842,7 +1849,7 @@ load_sort_win (WINDOW * main_win, GModule module, GSort * sort)
 
 /* Help menu data (F1/h). */
 static const char *help_main[] = {
-  "Copyright (C) 2009-2016 by Gerardo Orellana",
+  "Copyright (C) 2009-2017 by Gerardo Orellana",
   "http://goaccess.io - <hello@goaccess.io>",
   "Released under the MIT License.",
   "",
@@ -1890,10 +1897,6 @@ static const char *help_main[] = {
   "If you believe you have found a bug, please drop me",
   "an email with details.",
   "",
-  "If you have a medium or high traffic website, it",
-  "would be interesting to hear your experience with",
-  "GoAccess, such as generating time, visitors or hits.",
-  "",
   "Feedback? Just shoot me an email to:",
   "hello@goaccess.io",
 };
@@ -1930,8 +1933,8 @@ load_help_popup (WINDOW * main_win)
   }
   post_gmenu (menu);
 
-  draw_header (win, "GoAccess Quick Help", " %s", 1, 1, w2, color_panel_header);
-  mvwprintw (win, 2, 2, "[UP/DOWN] to scroll - [q] to quit");
+  draw_header (win, HELPDLG_HEAD, " %s", 1, 1, w2, color_panel_header);
+  mvwprintw (win, 2, 2, HELPDLG_DESC);
 
   wrefresh (win);
   while (quit) {
