@@ -44,6 +44,9 @@
 #include <pthread.h>
 #endif
 
+/* string literals and translations */
+#include "labels.h"
+
 /* Global UI defaults */
 #define MIN_HEIGHT        8     /* minimum window height */
 #define MIN_WIDTH         0     /* minimum window width */
@@ -51,109 +54,28 @@
 #define MAX_HEIGHT_HEADER 7     /* height of the header window */
 #define OVERALL_NUM_COLS  4     /* number of columns on the overall stats win */
 
-/* Overall Stats Labels */
-#define T_DASH       "Dashboard"
-#define T_HEAD       "Overall Analyzed Requests"
-
-#define T_DATETIME   "Date/Time"
-#define T_REQUESTS   "Total Requests"
-#define T_GEN_TIME   "Init. Proc. Time"
-#define T_FAILED     "Failed Requests"
-#define T_VALID      "Valid Requests"
-#define T_UNIQUE_VIS "Unique Visitors"
-#define T_UNIQUE_FIL "Unique Files"
-#define T_EXCLUDE_IP "Excl. IP Hits"
-#define T_REFERRER   "Referrers"
-#define T_UNIQUE404  "Unique 404"
-#define T_STATIC_FIL "Static Files"
-#define T_LOG        "Log Size"
-#define T_BW         "Bandwidth"
-#define T_LOG_PATH   "Log Source"
-
 /* Spinner Label Format */
 #define SPIN_FMT "%s"
 #define SPIN_FMTM "%s [%'d] [%'lld/s]"
 #define SPIN_LBL 50     /* max length of the progress spinner */
 
-#define INCLUDE_BOTS " - Including spiders"
-
-/* Module Labels and Descriptions */
-#define VISIT_HEAD  "Unique visitors per day"
-#define VISIT_DESC  "Hits having the same IP, date and agent are a unique visit."
-#define VISIT_ID    "visitors"
-#define VISIT_LABEL "Visitors"
-
-#define REQUE_HEAD  "Requested Files (URLs)"
-#define REQUE_DESC  "Top requests sorted by hits [, avgts, cumts, maxts, mthd, proto]"
-#define REQUE_ID    "requests"
-#define REQUE_LABEL "Requests"
-
-#define STATI_HEAD  "Static Requests"
-#define STATI_DESC  "Top static requests sorted by hits [, avgts, cumts, maxts, mthd, proto]"
-#define STATI_ID    "static_requests"
-#define STATI_LABEL "Static Requests"
-
-#define VTIME_HEAD  "Time Distribution"
-#define VTIME_DESC  "Data sorted by hour [, avgts, cumts, maxts]"
-#define VTIME_ID    "visit_time"
-#define VTIME_LABEL "Time"
-
-#define VHOST_HEAD  "Virtual Hosts"
-#define VHOST_DESC  "Data sorted by hits [, avgts, cumts, maxts]"
-#define VHOST_ID    "vhosts"
-#define VHOST_LABEL "Virtual Hosts"
-
-#define RUSER_HEAD  "Remote User (HTTP authentication)"
-#define RUSER_DESC  "Data sorted by hits [, avgts, cumts, maxts]"
-#define RUSER_ID    "remote_user"
-#define RUSER_LABEL "Remote User"
-
-#define FOUND_HEAD  "Not Found URLs (404s)"
-#define FOUND_DESC  "Top not found URLs sorted by hits [, avgts, cumts, maxts, mthd, proto]"
-#define FOUND_ID    "not_found"
-#define FOUND_LABEL "Not Found"
-
-#define HOSTS_HEAD  "Visitor Hostnames and IPs"
-#define HOSTS_DESC  "Top visitor hosts sorted by hits [, avgts, cumts, maxts]"
-#define HOSTS_ID    "hosts"
-#define HOSTS_LABEL "Hosts"
-
-#define OPERA_HEAD  "Operating Systems"
-#define OPERA_DESC  "Top Operating Systems sorted by hits [, avgts, cumts, maxts]"
-#define OPERA_ID    "os"
-#define OPERA_LABEL "OS"
-
-#define BROWS_HEAD  "Browsers"
-#define BROWS_DESC  "Top Browsers sorted by hits [, avgts, cumts, maxts]"
-#define BROWS_ID    "browsers"
-#define BROWS_LABEL "Browsers"
-
-#define REFER_HEAD  "Referrers URLs"
-#define REFER_DESC  "Top Requested Referrers sorted by hits [, avgts, cumts, maxts]"
-#define REFER_ID    "referrers"
-#define REFER_LABEL "Referrers"
-
-#define SITES_HEAD  "Referring Sites"
-#define SITES_DESC  "Top Referring Sites sorted by hits [, avgts, cumts, maxts]"
-#define SITES_ID    "referring_sites"
-#define SITES_LABEL "Referring Sites"
-
-#define KEYPH_HEAD  "Keyphrases from Google's search engine"
-#define KEYPH_DESC  "Top Keyphrases sorted by hits [, avgts, cumts, maxts]"
-#define KEYPH_ID    "keyphrases"
-#define KEYPH_LABEL "Keyphrases"
-
-#define GEOLO_HEAD  "Geo Location"
-#define GEOLO_DESC  "Continent > Country sorted by unique hits [, avgts, cumts, maxts]"
-#define GEOLO_ID    "geolocation"
-#define GEOLO_LABEL "Geo Location"
-
-#define CODES_HEAD  "HTTP Status Codes"
-#define CODES_DESC  "Top HTTP Status Codes sorted by hits [, avgts, cumts, maxts]"
-#define CODES_ID    "status_codes"
-#define CODES_LABEL "Status Codes"
-
-#define GENER_ID   "general"
+/* Module JSON keys */
+#define VISITORS_ID        "visitors"
+#define REQUESTS_ID        "requests"
+#define REQUESTS_STATIC_ID "static_requests"
+#define VISIT_TIMES_ID     "visit_time"
+#define VIRTUAL_HOSTS_ID   "vhosts"
+#define REMOTE_USER_ID     "remote_user"
+#define NOT_FOUND_ID       "not_found"
+#define HOSTS_ID           "hosts"
+#define OS_ID              "os"
+#define BROWSERS_ID        "browsers"
+#define REFERRERS_ID       "referrers"
+#define REFERRING_SITES_ID "referring_sites"
+#define KEYPHRASES_ID      "keyphrases"
+#define GEO_LOCATION_ID    "geolocation"
+#define STATUS_CODES_ID    "status_codes"
+#define GENER_ID           "general"
 
 /* Overall Statistics CSV/JSON Keys */
 #define OVERALL_STARTDATE "start_date"
@@ -173,27 +95,6 @@
 #define OVERALL_BANDWIDTH "bandwidth"
 #define OVERALL_LOG       "log_path"
 
-/* Metric Labels */
-#define MTRC_HITS_LBL            "Hits"
-#define MTRC_VISITORS_LBL        "Visitors"
-#define MTRC_VISITORS_SHORT_LBL  "Vis."
-#define MTRC_BW_LBL              "Bandwidth"
-#define MTRC_AVGTS_LBL           "Avg. T.S."
-#define MTRC_CUMTS_LBL           "Cum. T.S."
-#define MTRC_MAXTS_LBL           "Max. T.S."
-#define MTRC_METHODS_LBL         "Method"
-#define MTRC_METHODS_SHORT_LBL   "Mtd"
-#define MTRC_PROTOCOLS_LBL       "Protocol"
-#define MTRC_PROTOCOLS_SHORT_LBL "Proto"
-#define MTRC_CITY_LBL            "City"
-#define MTRC_COUNTRY_LBL         "Country"
-#define MTRC_HOSTNAME_LBL        "Hostname"
-#define MTRC_DATA_LBL            "Data"
-
-/* Find Labels */
-#define FIND_HEAD    "Find pattern in all views"
-#define FIND_DESC    "Regex allowed - ^g to cancel - TAB switch case"
-
 /* CONFIG DIALOG */
 #define CONF_MENU_H       6
 #define CONF_MENU_W       57
@@ -222,8 +123,6 @@
 #define SORT_MENU_Y       4
 #define SORT_WIN_H        13
 #define SORT_WIN_W        42
-#define SORT_ASC_SEL      "[x] ASC [ ] DESC"
-#define SORT_DESC_SEL     "[ ] ASC [x] DESC"
 
 /* AGENTS DIALOG */
 #define AGENTS_MENU_X     2
@@ -238,16 +137,12 @@
 #define HELP_WIN_WIDTH    64
 
 /* CONF ERROR DIALOG */
-#define ERR_MENU_HEIGHT  10
-#define ERR_MENU_WIDTH   57
-#define ERR_MENU_X       2
-#define ERR_MENU_Y       4
-#define ERR_WIN_HEIGHT   15
-#define ERR_WIN_WIDTH    61
-#define ERR_HEADER "Format Errors - Verify your log/date/time format"
-
-#define CSENSITIVE   "[x] case sensitive"
-#define CISENSITIVE  "[ ] case sensitive"
+#define ERR_MENU_HEIGHT   10
+#define ERR_MENU_WIDTH    57
+#define ERR_MENU_X        2
+#define ERR_MENU_Y        4
+#define ERR_WIN_HEIGHT    15
+#define ERR_WIN_WIDTH     61
 
 /* Convenient macros */
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
@@ -357,6 +252,6 @@ void term_size (WINDOW * main_win, int *main_win_height);
 void ui_spinner_create (GSpinner * spinner);
 void update_active_module (WINDOW * header_win, GModule current);
 void update_header (WINDOW * header_win, int current);
-
 /* *INDENT-ON* */
+
 #endif
