@@ -54,72 +54,82 @@
 #include "util.h"
 
 #include "error.h"
+#include "labels.h"
 #include "xmalloc.h"
 
 /* HTTP status codes categories */
 static const char *code_type[][2] = {
-  {"1", "1xx Informational"},
-  {"2", "2xx Success"},
-  {"3", "3xx Redirection"},
-  {"4", "4xx Client Error"},
-  {"5", "5xx Server Error"},
+  {"1", STATUS_CODE_1XX},
+  {"2", STATUS_CODE_2XX},
+  {"3", STATUS_CODE_3XX},
+  {"4", STATUS_CODE_4XX},
+  {"5", STATUS_CODE_5XX},
 };
 
 /* HTTP status codes */
 static const char *codes[][2] = {
-  {"100", "100 - Continue: Server received the initial part of the request"},
-  {"101", "101 - Switching Protocols: Client asked to switch protocols"},
-  {"200", "200 - OK: The request sent by the client was successful"},
-  {"201", "201 - Created: The request has been fulfilled and created"},
-  {"202", "202 - Accepted: The request has been accepted for processing"},
-  {"203", "203 - Non-authoritative Information: Response from a third party"},
-  {"204", "204 - No Content: Request did not return any content"},
-  {"205", "205 - Reset Content: Server asked the client to reset the document"},
-  {"206", "206 - Partial Content: The partial GET has been successful"},
-  {"207", "207 - Multi-Status: WebDAV; RFC 4918"},
-  {"208", "208 - Already Reported: WebDAV; RFC 5842"},
-  {"300", "300 - Multiple Choices: Multiple options for the resource"},
-  {"301", "301 - Moved Permanently: Resource has permanently moved"},
-  {"302", "302 - Moved Temporarily (redirect)"},
-  {"303", "303 - See Other Document: The response is at a different URI"},
-  {"304", "304 - Not Modified: Resource has not been modified"},
-  {"305", "305 - Use Proxy: Can only be accessed through the proxy"},
-  {"307", "307 - Temporary Redirect: Resource temporarily moved"},
-  {"400", "400 - Bad Request: The syntax of the request is invalid"},
-  {"401", "401 - Unauthorized: Request needs user authentication"},
-  {"402", "402 - Payment Required"},
-  {"403", "403 - Forbidden: Server is refusing to respond to it"},
-  {"404", "404 - Not Found: Requested resource could not be found"},
-  {"405", "405 - Method Not Allowed: Request method not supported"},
-  {"406", "406 - Not Acceptable"},
-  {"407", "407 - Proxy Authentication Required"},
-  {"408", "408 - Request Timeout: Server timed out waiting for the request"},
-  {"409", "409 - Conflict: Conflict in the request"},
-  {"410", "410 - Gone: Resource requested is no longer available"},
-  {"411", "411 - Length Required: Invalid Content-Length"},
-  {"412", "412 - Precondition Failed: Server does not meet preconditions"},
-  {"413", "413 - Payload Too Large"},
-  {"414", "414 - Request-URI Too Long"},
-  {"415", "415 - Unsupported Media Type: Media type is not supported"},
-  {"416", "416 - Requested Range Not Satisfiable: Cannot supply that portion"},
-  {"417", "417 - Expectation Failed"},
-  {"444", "444 - (Nginx) Connection closed without sending any headers"},
-  {"494", "494 - (Nginx) Request Header Too Large"},
-  {"495", "495 - (Nginx) SSL client certificate error"},
-  {"496", "496 - (Nginx) Client didn't provide certificate"},
-  {"497", "497 - (Nginx) HTTP request sent to HTTPS port"},
-  {"499", "499 - (Nginx) Connection closed by client while processing request"},
-  {"500", "500 - Internal Server Error"},
-  {"501", "501 - Not Implemented"},
-  {"502", "502 - Bad Gateway: Received an invalid response from the upstream"},
-  {"503", "503 - Service Unavailable: The server is currently unavailable"},
-  {"504", "504 - Gateway Timeout: The upstream server failed to send request"},
-  {"505", "505 - HTTP Version Not Supported"},
-  {"520", "520 - CloudFlare - Web server is returning an unknown error"},
-  {"521", "521 - CloudFlare - Web server is down"},
-  {"522", "522 - CloudFlare - Connection timed out"},
-  {"523", "523 - CloudFlare - Origin is unreachable"},
-  {"524", "524 - CloudFlare - A timeout occured"}
+  {"100", STATUS_CODE_100},
+  {"101", STATUS_CODE_101},
+  {"200", STATUS_CODE_200},
+  {"201", STATUS_CODE_201},
+  {"202", STATUS_CODE_202},
+  {"203", STATUS_CODE_203},
+  {"204", STATUS_CODE_204},
+  {"205", STATUS_CODE_205},
+  {"206", STATUS_CODE_206},
+  {"207", STATUS_CODE_207},
+  {"208", STATUS_CODE_208},
+  {"300", STATUS_CODE_300},
+  {"301", STATUS_CODE_301},
+  {"302", STATUS_CODE_302},
+  {"303", STATUS_CODE_303},
+  {"304", STATUS_CODE_304},
+  {"305", STATUS_CODE_305},
+  {"307", STATUS_CODE_307},
+  {"400", STATUS_CODE_400},
+  {"401", STATUS_CODE_401},
+  {"402", STATUS_CODE_402},
+  {"403", STATUS_CODE_403},
+  {"404", STATUS_CODE_404},
+  {"405", STATUS_CODE_405},
+  {"406", STATUS_CODE_406},
+  {"407", STATUS_CODE_407},
+  {"408", STATUS_CODE_408},
+  {"409", STATUS_CODE_409},
+  {"410", STATUS_CODE_410},
+  {"411", STATUS_CODE_411},
+  {"412", STATUS_CODE_412},
+  {"413", STATUS_CODE_413},
+  {"414", STATUS_CODE_414},
+  {"415", STATUS_CODE_415},
+  {"416", STATUS_CODE_416},
+  {"417", STATUS_CODE_417},
+  {"421", STATUS_CODE_421},
+  {"422", STATUS_CODE_422},
+  {"423", STATUS_CODE_423},
+  {"424", STATUS_CODE_424},
+  {"426", STATUS_CODE_426},
+  {"428", STATUS_CODE_428},
+  {"429", STATUS_CODE_429},
+  {"431", STATUS_CODE_431},
+  {"444", STATUS_CODE_444},
+  {"451", STATUS_CODE_451},
+  {"494", STATUS_CODE_494},
+  {"495", STATUS_CODE_495},
+  {"496", STATUS_CODE_496},
+  {"497", STATUS_CODE_497},
+  {"499", STATUS_CODE_499},
+  {"500", STATUS_CODE_500},
+  {"501", STATUS_CODE_501},
+  {"502", STATUS_CODE_502},
+  {"503", STATUS_CODE_503},
+  {"504", STATUS_CODE_504},
+  {"505", STATUS_CODE_505},
+  {"520", STATUS_CODE_520},
+  {"521", STATUS_CODE_521},
+  {"522", STATUS_CODE_522},
+  {"523", STATUS_CODE_523},
+  {"524", STATUS_CODE_524}
 };
 
 /* Return part of a string
@@ -174,6 +184,17 @@ xstrncpy (char *dest, const char *source, const size_t dest_size)
   }
 }
 
+/* A random string generator. */
+void
+genstr (char *dest, size_t len)
+{
+  char set[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
+  while (len-- > 0)
+    *dest++ = set[rand () % (sizeof (set) - 1)];
+  *dest = '\0';
+}
+
 /* Count the number of matches on the string `s1` given a character `c`
  *
  * If the character is not found, 0 is returned
@@ -250,6 +271,38 @@ out:
   return ignore;
 }
 
+/* Determine if the given host needs to be hidden given the list of
+ * referrers to hide.
+ *
+ * On error, or the referrer is not found, 0 is returned
+ * On success, or if the host needs to be ignored, 1 is returned */
+int
+hide_referer (const char *host)
+{
+  char *needle = NULL;
+  int i, ignore = 0;
+
+  if (conf.hide_referer_idx == 0)
+    return 0;
+  if (host == NULL || *host == '\0')
+    return 0;
+
+  needle = xstrdup (host);
+  for (i = 0; i < conf.hide_referer_idx; ++i) {
+    if (conf.hide_referers[i] == NULL || *conf.hide_referers[i] == '\0')
+      continue;
+
+    if (wc_match (conf.hide_referers[i], needle)) {
+      ignore = 1;
+      goto out;
+    }
+  }
+out:
+  free (needle);
+
+  return ignore;
+}
+
 /* Determine if the given ip is within a range of IPs.
  *
  * On error, or not within the range, 0 is returned
@@ -361,6 +414,43 @@ find_output_type (char **filename, const char *ext, int alloc)
   return 1;
 }
 
+/* Validates the '-o' filename extension for any of:
+ * 1) .csv
+ * 2) .json
+ * 3) .html
+ *
+ * Return Value
+ * 1: valid
+ * 0: invalid
+ * -1: non-existent extension
+ */
+int
+valid_output_type (const char *filename)
+{
+  const char *ext = NULL;
+  size_t sl;
+
+  if ((ext = strrchr (filename, '.')) == NULL)
+    return -1;
+
+  ext++;
+  /* Is extension 3<=len<=4? */
+  sl = strlen (ext);
+  if (sl < 3 || sl > 4)
+    return 0;
+
+  if (strcmp ("html", ext) == 0)
+    return 1;
+
+  if (strcmp ("json", ext) == 0)
+    return 1;
+
+  if (strcmp ("csv", ext) == 0)
+    return 1;
+
+  return 0;
+}
+
 /* Search the environment HOME variable and append GoAccess' config
  * file.
  *
@@ -539,7 +629,7 @@ verify_status_code_type (const char *str)
   size_t i;
   for (i = 0; i < ARRAY_SIZE (code_type); i++)
     if (strchr (code_type[i][0], str[0]) != NULL)
-      return code_type[i][1];
+      return _(code_type[i][1]);
 
   return "Unknown";
 }
@@ -555,7 +645,7 @@ verify_status_code (char *str)
   size_t i;
   for (i = 0; i < ARRAY_SIZE (codes); i++)
     if (strstr (str, codes[i][0]) != NULL)
-      return codes[i][1];
+      return _(codes[i][1]);
 
   return "Unknown";
 }
@@ -624,14 +714,16 @@ char *
 filesize_str (unsigned long long log_size)
 {
   char *size = xmalloc (sizeof (char) * 12);
-  if (log_size >= TIB)
-    snprintf (size, 12, "%.2f TiB", (double) (log_size) / TIB);
-  else if (log_size >= GIB)
-    snprintf (size, 12, "%.2f GiB", (double) (log_size) / GIB);
-  else if (log_size >= MIB)
-    snprintf (size, 12, "%.2f MiB", (double) (log_size) / MIB);
-  else if (log_size >= KIB)
-    snprintf (size, 12, "%.2f KiB", (double) (log_size) / KIB);
+  if (log_size >= (1ULL << 50))
+    snprintf (size, 12, "%.2f PiB", (double) (log_size) / PIB (1ULL));
+  else if (log_size >= (1ULL << 40))
+    snprintf (size, 12, "%.2f TiB", (double) (log_size) / TIB (1ULL));
+  else if (log_size >= (1ULL << 30))
+    snprintf (size, 12, "%.2f GiB", (double) (log_size) / GIB (1ULL));
+  else if (log_size >= (1ULL << 20))
+    snprintf (size, 12, "%.2f MiB", (double) (log_size) / MIB (1ULL));
+  else if (log_size >= (1ULL << 10))
+    snprintf (size, 12, "%.2f KiB", (double) (log_size) / KIB (1ULL));
   else
     snprintf (size, 12, "%.1f   B", (double) (log_size));
 
