@@ -34,10 +34,11 @@
 
 #define REGEX_ERROR 100
 
-#define KIB 1024
-#define MIB 1048576
-#define GIB 1073741824
-#define TIB 1099511627776LL
+#define KIB(n) (n << 10)
+#define MIB(n) (n << 20)
+#define GIB(n) (n << 30)
+#define TIB(n) (n << 40)
+#define PIB(n) (n << 50)
 
 #define MILS 1000ULL
 #define SECS 1000000ULL
@@ -76,15 +77,18 @@ const char *verify_status_code_type (const char *str);
 int convert_date (char *res, const char *data, const char *from, const char *to, int size);
 int count_matches (const char *s1, char c);
 int find_output_type (char **filename, const char *ext, int alloc);
+int hide_referer (const char *ref);
 int ignore_referer (const char *ref);
 int intlen (int num);
 int invalid_ipaddr (char *str, int *ipvx);
 int ip_in_range (const char *ip);
 int str_inarray (const char *s, const char *arr[], int size);
 int str_to_time (const char *str, const char *fmt, struct tm *tm);
+int valid_output_type (const char *filename);
 off_t file_size (const char *filename);
 uint32_t ip_to_binary (const char *ip);
 void append_str (char **dest, const char *src);
+void genstr(char *dest, size_t len);
 void strip_newlines (char *str);
 void xstrncpy (char *dest, const char *source, const size_t dest_size);
 
