@@ -498,7 +498,9 @@ ws_ssl_cleanup (WSServer * server)
   CRYPTO_set_id_callback (NULL);
   CRYPTO_set_locking_callback (NULL);
   ERR_free_strings ();
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
   ERR_remove_state (0);
+#endif
   EVP_cleanup ();
 }
 #endif
@@ -571,7 +573,7 @@ ws_clear_pipeout (WSPipeOut * pipeout)
     unlink (wsconfig.pipeout);
 }
 
-/* Stop the server and do some clearning. */
+/* Stop the server and do some cleaning. */
 void
 ws_stop (WSServer * server)
 {
@@ -644,7 +646,11 @@ initialize_ssl_ctx (WSServer * server)
   int ret = 1;
   SSL_CTX *ctx = NULL;
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+  SSL_library_init ();
   SSL_load_error_strings ();
+#endif
+
   /* Ciphers and message digests */
   OpenSSL_add_ssl_algorithms ();
 
@@ -680,35 +686,48 @@ out:
 static void
 log_return_message (int ret, int err, const char *fn)
 {
+  unsigned long e;
+
   switch (err) {
   case SSL_ERROR_NONE:
-    LOG (("SSL: %s - SSL_ERROR_NONE\n", fn));
+    LOG (("SSL: %s -> SSL_ERROR_NONE\n", fn));
     LOG (("SSL: TLS/SSL I/O operation completed\n"));
     break;
   case SSL_ERROR_WANT_READ:
-    LOG (("SSL: %s - SSL_ERROR_WANT_READ\n", fn));
+    LOG (("SSL: %s -> SSL_ERROR_WANT_READ\n", fn));
     LOG (("SSL: incomplete, data available for reading\n"));
     break;
   case SSL_ERROR_WANT_WRITE:
-    LOG (("SSL: %s - SSL_ERROR_WANT_WRITE\n", fn));
+    LOG (("SSL: %s -> SSL_ERROR_WANT_WRITE\n", fn));
     LOG (("SSL: incomplete, data available for writing\n"));
     break;
   case SSL_ERROR_ZERO_RETURN:
-    LOG (("SSL: %s - SSL_ERROR_ZERO_RETURN\n", fn));
+    LOG (("SSL: %s -> SSL_ERROR_ZERO_RETURN\n", fn));
     LOG (("SSL: TLS/SSL connection has been closed\n"));
     break;
   case SSL_ERROR_WANT_X509_LOOKUP:
-    LOG (("SSL: %s - SSL_ERROR_WANT_X509_LOOKUP\n", fn));
+    LOG (("SSL: %s -> SSL_ERROR_WANT_X509_LOOKUP\n", fn));
     break;
   case SSL_ERROR_SYSCALL:
-    LOG (("SSL: %s - SSL_ERROR_SYSCALL\n", fn));
-    if (ret >= 0)
-      LOG (("SSL: handshake interrupted, got EOF\n"));
-    else
+    LOG (("SSL: %s -> SSL_ERROR_SYSCALL\n", fn));
+
+    e = ERR_get_error ();
+    if (e > 0)
+      LOG (("SSL: %s -> %s\n", fn, ERR_error_string (e, NULL)));
+
+    /* call was not successful because a fatal error occurred either at the
+     * protocol level or a connection failure occurred. */
+    if (ret != 0) {
       LOG (("SSL bogus handshake interrupt: \n", strerror (errno)));
+      break;
+    }
+    /* call not yet finished. */
+    LOG (("SSL: handshake interrupted, got EOF\n"));
+    if (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN)
+      LOG (("SSL: %s -> not yet finished %s\n", fn, strerror (errno)));
     break;
   default:
-    LOG (("SSL: %s - failed fatal error code: %d\n", fn, err));
+    LOG (("SSL: %s -> failed fatal error code: %d\n", fn, err));
     LOG (("SSL: %s\n", ERR_error_string (ERR_get_error (), NULL)));
     break;
   }
@@ -737,10 +756,14 @@ shutdown_ssl (WSClient * client)
     client->sslstatus = WS_TLS_SHUTTING;
     break;
   case SSL_ERROR_SYSCALL:
-    if (ret == 0)
+    if (ret == 0) {
       LOG (("SSL: SSL_shutdown, connection unexpectedly closed by peer.\n"));
-    else
-      LOG (("SSL: SSL_shutdown, probably unrecoverable, forcing close.\n"));
+      /* The shutdown is not yet finished. */
+      if (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN)
+        client->sslstatus = WS_TLS_SHUTTING;
+      break;
+    }
+    LOG (("SSL: SSL_shutdown, probably unrecoverable, forcing close.\n"));
   case SSL_ERROR_ZERO_RETURN:
   case SSL_ERROR_WANT_X509_LOOKUP:
   default:
@@ -775,8 +798,14 @@ accept_ssl (WSClient * client)
     client->sslstatus = WS_TLS_ACCEPTING;
     break;
   case SSL_ERROR_SYSCALL:
-    if (ret < 0 && (errno == EAGAIN || errno == EWOULDBLOCK))
+    /* Wait for more activity else bail out, for instance if the socket is closed
+     * during the handshake. */
+    if (ret < 0 && (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR)) {
+      client->sslstatus = WS_TLS_ACCEPTING;
       break;
+    }
+    /* The peer notified that it is shutting down through a SSL "close_notify" so
+     * we shutdown too */
   case SSL_ERROR_ZERO_RETURN:
   case SSL_ERROR_WANT_X509_LOOKUP:
   default:
@@ -854,7 +883,9 @@ send_ssl_buffer (WSClient * client, const char *buffer, int len)
 {
   int bytes = 0, err = 0;
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
   ERR_clear_error ();
+#endif
   if ((bytes = SSL_write (client->ssl, buffer, len)) > 0)
     return bytes;
 
@@ -868,8 +899,10 @@ send_ssl_buffer (WSClient * client, const char *buffer, int len)
     client->sslstatus = WS_TLS_WRITING;
     break;
   case SSL_ERROR_SYSCALL:
-    if ((bytes < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)))
+    if ((bytes < 0 &&
+         (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR)))
       break;
+    /* The connection was shut down cleanly */
   case SSL_ERROR_ZERO_RETURN:
   case SSL_ERROR_WANT_X509_LOOKUP:
   default:
@@ -889,7 +922,9 @@ read_ssl_socket (WSClient * client, char *buffer, int size)
 {
   int bytes = 0, done = 0, err = 0;
   do {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     ERR_clear_error ();
+#endif
 
     done = 0;
     if ((bytes = SSL_read (client->ssl, buffer, size)) > 0)
@@ -907,7 +942,8 @@ read_ssl_socket (WSClient * client, char *buffer, int size)
       done = 1;
       break;
     case SSL_ERROR_SYSCALL:
-      if ((bytes < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)))
+      if ((bytes < 0 &&
+           (errno == EAGAIN || errno == EWOULDBLOCK || errno == EINTR)))
         break;
     case SSL_ERROR_ZERO_RETURN:
     case SSL_ERROR_WANT_X509_LOOKUP:
@@ -987,8 +1023,8 @@ ws_get_method (const char *token)
 {
   const char *lookfor = NULL;
 
-  if ((lookfor = "GET", !memcmp (token, lookfor, 3)) ||
-      (lookfor = "get", !memcmp (token, lookfor, 3)))
+  if ((lookfor = "GET", !memcmp (token, "GET ", 4)) ||
+      (lookfor = "get", !memcmp (token, "get ", 4)))
     return lookfor;
   return NULL;
 }
@@ -1094,7 +1130,7 @@ ws_set_header_fields (char *line, WSHeaders * headers)
   if (line[0] == '\n' || line[0] == '\r')
     return 1;
 
-  if ((strstr (line, "GET")) || (strstr (line, "get"))) {
+  if ((strstr (line, "GET ")) || (strstr (line, "get "))) {
     if ((path = ws_parse_request (line, &method, &proto)) == NULL)
       return 1;
     headers->path = path;
@@ -2823,13 +2859,14 @@ ws_start (WSServer * server)
   if (wsconfig.sslcert && wsconfig.sslkey) {
     LOG (("==Using TLS/SSL==\n"));
     wsconfig.use_ssl = 1;
-    if (initialize_ssl_ctx (server))
+    if (initialize_ssl_ctx (server)) {
+      LOG (("Unable to initialize_ssl_ctx\n"));
       return;
+    }
   }
 #endif
 
   memset (&fdstate, 0, sizeof fdstate);
-  ws_fifo (server);
   ws_socket (&listener);
 
   while (1) {
@@ -2849,14 +2886,17 @@ ws_start (WSServer * server)
     if (select (max_file_fd, &fdstate.rfds, &fdstate.wfds, NULL, NULL) == -1) {
       switch (errno) {
       case EINTR:
+        LOG (("A signal was caught on select(2)\n"));
         break;
       default:
         FATAL ("Unable to select: %s.", strerror (errno));
       }
     }
     /* handle self-pipe trick */
-    if (FD_ISSET (server->self_pipe[0], &fdstate.rfds))
+    if (FD_ISSET (server->self_pipe[0], &fdstate.rfds)) {
+      LOG (("Handled self-pipe to close event loop.\n"));
       break;
+    }
 
     /* iterate over existing connections */
     for (conn = 0; conn < max_file_fd; ++conn) {
@@ -2952,7 +2992,7 @@ ws_set_config_sslkey (const char *sslkey)
 
 /* Create a new websocket server context. */
 WSServer *
-ws_init (const char *host, const char *port)
+ws_init (const char *host, const char *port, void (*initopts) (void))
 {
   WSServer *server = new_wsserver ();
   server->pipein = new_wspipein ();
@@ -2971,5 +3011,8 @@ ws_init (const char *host, const char *port)
   wsconfig.strict = 0;
   wsconfig.use_ssl = 0;
 
+  initopts ();
+  ws_fifo (server);
+
   return server;
 }
